/* valasemanticanalyzer.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valasemanticanalyzer.h"
#include <vala/valasemanticanalyzer.h>
#include <vala/valareport.h>
#include <vala/valasourcereference.h>
#include <vala/valamemberaccessibility.h>
#include <vala/valastatement.h>
#include <vala/valacallback.h>
#include <vala/valaarray.h>
#include <vala/valalockable.h>
#include <vala/valatypeparameter.h>
#include <vala/valaliteral.h>
#include <vala/valaenumvalue.h>
#include <vala/valanamespacereference.h>
#include <vala/valaenum.h>
#include <vala/valainvokable.h>
#include <vala/valasignal.h>

struct _ValaSemanticAnalyzerPrivate {
	gboolean _memory_management;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaSourceFile* current_source_file;
	ValaTypeReference* current_return_type;
	ValaClass* current_class;
	ValaStruct* current_struct;
	GList* current_using_directives;
	ValaTypeReference* bool_type;
	ValaTypeReference* string_type;
	ValaTypeReference* int_type;
	ValaTypeReference* type_type;
	ValaDataType* pointer_type;
	ValaDataType* initially_unowned_type;
	int next_lambda_id;
};
#define VALA_SEMANTIC_ANALYZER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerPrivate))
enum  {
	VALA_SEMANTIC_ANALYZER_DUMMY_PROPERTY,
	VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT,
};
static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_semantic_analyzer_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_semantic_analyzer_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_find_base_class_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl);
static void vala_semantic_analyzer_find_base_interface_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_real_visit_begin_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
static void vala_semantic_analyzer_real_visit_end_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n);
static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_semantic_analyzer_real_visit_begin_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list);
static void vala_semantic_analyzer_real_visit_end_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list);
static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_semantic_analyzer_real_visit_end_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_semantic_analyzer_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt);
static void vala_semantic_analyzer_real_visit_begin_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_end_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr);
static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr);
static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr);
static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr);
static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr);
static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr);
static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr);
static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node);
static ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSemanticAnalyzer* self, ValaSymbol* sym, char* name);
static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr);
static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym);
static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type);
static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static gboolean vala_semantic_analyzer_check_arguments (ValaSemanticAnalyzer* self, ValaExpression* expr, ValaSymbol* msym, GList* params, GList* args);
static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr);
static void vala_semantic_analyzer_real_visit_base_access (ValaCodeVisitor* base, ValaBaseAccess* expr);
static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr);
static void vala_semantic_analyzer_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr);
static gboolean vala_semantic_analyzer_is_numeric_type (ValaSemanticAnalyzer* self, ValaTypeReference* type);
static gboolean vala_semantic_analyzer_is_integer_type (ValaSemanticAnalyzer* self, ValaTypeReference* type);
static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr);
static ValaMemberAccess* vala_semantic_analyzer_find_member_access (ValaSemanticAnalyzer* self, ValaExpression* expr);
static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr);
static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation);
static ValaTypeReference* vala_semantic_analyzer_get_arithmetic_result_type (ValaSemanticAnalyzer* self, ValaTypeReference* left_type, ValaTypeReference* right_type);
static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr);
static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr);
static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self);
static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self);
static gboolean vala_semantic_analyzer_is_in_constructor (ValaSemanticAnalyzer* self);
static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l);
static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static gpointer vala_semantic_analyzer_parent_class = NULL;
static void vala_semantic_analyzer_dispose (GObject * obj);


ValaSemanticAnalyzer* vala_semantic_analyzer_new (gboolean manage_memory)
{
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "memory-management", g_value_init (&__params_it->value, G_TYPE_BOOLEAN), g_value_set_boolean (&__params_it->value, manage_memory), __params_it++);
	ValaSemanticAnalyzer * self = g_object_newv (VALA_TYPE_SEMANTIC_ANALYZER, __params_it - __params, __params);
	return self;
}


/**
 * Analyze and check code in the specified context.
 *
 * @param context a code context
 */
void vala_semantic_analyzer_analyze (ValaSemanticAnalyzer* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaSymbol* __temp0 = NULL;
	self->priv->root_symbol = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp0);
	ValaTypeReference* __temp1 = NULL;
	self->priv->bool_type = (__temp1 = vala_type_reference_new (), (self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL))), __temp1);
	(vala_type_reference_set_data_type (self->priv->bool_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "bool")))), vala_type_reference_get_data_type (self->priv->bool_type));
	ValaTypeReference* __temp2 = NULL;
	self->priv->string_type = (__temp2 = vala_type_reference_new (), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), __temp2);
	(vala_type_reference_set_data_type (self->priv->string_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "string")))), vala_type_reference_get_data_type (self->priv->string_type));
	ValaDataType* __temp4 = NULL;
	ValaDataType* __temp3 = NULL;
	self->priv->pointer_type = (__temp4 = (__temp3 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "pointer"))), (__temp3 == NULL ? NULL : g_object_ref (__temp3))), (self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL))), __temp4);
	ValaTypeReference* __temp5 = NULL;
	self->priv->int_type = (__temp5 = vala_type_reference_new (), (self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL))), __temp5);
	(vala_type_reference_set_data_type (self->priv->int_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")))), vala_type_reference_get_data_type (self->priv->int_type));
	/* TODO: don't require GLib namespace in semantic analyzer*/
	ValaSymbol* glib_ns = vala_symbol_lookup (self->priv->root_symbol, "GLib");
	if (glib_ns != NULL) {
		ValaDataType* __temp7 = NULL;
		ValaDataType* __temp6 = NULL;
		self->priv->initially_unowned_type = (__temp7 = (__temp6 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (glib_ns, "InitiallyUnowned"))), (__temp6 == NULL ? NULL : g_object_ref (__temp6))), (self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL))), __temp7);
		ValaTypeReference* __temp8 = NULL;
		self->priv->type_type = (__temp8 = vala_type_reference_new (), (self->priv->type_type == NULL ? NULL : (self->priv->type_type = (g_object_unref (self->priv->type_type), NULL))), __temp8);
		(vala_type_reference_set_data_type (self->priv->type_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (glib_ns, "Type")))), vala_type_reference_get_data_type (self->priv->type_type));
	}
	ValaSymbol* __temp10 = NULL;
	ValaSymbol* __temp9 = NULL;
	self->priv->current_symbol = (__temp10 = (__temp9 = self->priv->root_symbol, (__temp9 == NULL ? NULL : g_object_ref (__temp9))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp10);
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	ValaSourceFile* __temp11 = NULL;
	self->priv->current_source_file = (__temp11 = g_object_ref (file), (self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL))), __temp11);
	GList* __temp12 = NULL;
	self->priv->current_using_directives = (__temp12 = vala_source_file_get_using_directives (file), (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp12);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	GList* __temp13 = NULL;
	self->priv->current_using_directives = (__temp13 = NULL, (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp13);
}


static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp15 = NULL;
	ValaSymbol* __temp14 = NULL;
	self->priv->current_symbol = (__temp15 = (__temp14 = vala_code_node_get_symbol (VALA_CODE_NODE (ns)), (__temp14 == NULL ? NULL : g_object_ref (__temp14))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp15);
}


static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp17 = NULL;
	ValaSymbol* __temp16 = NULL;
	self->priv->current_symbol = (__temp17 = (__temp16 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp16 == NULL ? NULL : g_object_ref (__temp16))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp17);
}


static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp19 = NULL;
	ValaSymbol* __temp18 = NULL;
	self->priv->current_symbol = (__temp19 = (__temp18 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp18 == NULL ? NULL : g_object_ref (__temp18))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp19);
	ValaClass* __temp20 = NULL;
	self->priv->current_class = (__temp20 = g_object_ref (cl), (self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL))), __temp20);
	if (vala_class_get_base_class (cl) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_base_class (cl))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
	}
	{
		GList* __temp21 = NULL;
		__temp21 = vala_class_get_base_types (cl);
		GList* base_type_reference_it;
		for (base_type_reference_it = __temp21; base_type_reference_it != NULL; base_type_reference_it = base_type_reference_it->next) {
			ValaTypeReference* base_type_reference = base_type_reference_it->data;
			{
				vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (base_type_reference))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
			}
		}
		(__temp21 == NULL ? NULL : (__temp21 = (g_list_free (__temp21), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp23 = NULL;
	ValaSymbol* __temp22 = NULL;
	self->priv->current_symbol = (__temp23 = (__temp22 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp22 == NULL ? NULL : g_object_ref (__temp22))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp23);
	ValaClass* __temp24 = NULL;
	self->priv->current_class = (__temp24 = NULL, (self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL))), __temp24);
}


static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp26 = NULL;
	ValaSymbol* __temp25 = NULL;
	self->priv->current_symbol = (__temp26 = (__temp25 = vala_code_node_get_symbol (VALA_CODE_NODE (st)), (__temp25 == NULL ? NULL : g_object_ref (__temp25))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp26);
	ValaStruct* __temp27 = NULL;
	self->priv->current_struct = (__temp27 = g_object_ref (st), (self->priv->current_struct == NULL ? NULL : (self->priv->current_struct = (g_object_unref (self->priv->current_struct), NULL))), __temp27);
}


static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp29 = NULL;
	ValaSymbol* __temp28 = NULL;
	self->priv->current_symbol = (__temp29 = (__temp28 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp28 == NULL ? NULL : g_object_ref (__temp28))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp29);
	ValaStruct* __temp30 = NULL;
	self->priv->current_struct = (__temp30 = NULL, (self->priv->current_struct == NULL ? NULL : (self->priv->current_struct = (g_object_unref (self->priv->current_struct), NULL))), __temp30);
}


static void vala_semantic_analyzer_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp32 = NULL;
	ValaSymbol* __temp31 = NULL;
	self->priv->current_symbol = (__temp32 = (__temp31 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp31 == NULL ? NULL : g_object_ref (__temp31))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp32);
	{
		GList* __temp33 = NULL;
		__temp33 = vala_interface_get_prerequisites (iface);
		GList* prerequisite_reference_it;
		for (prerequisite_reference_it = __temp33; prerequisite_reference_it != NULL; prerequisite_reference_it = prerequisite_reference_it->next) {
			ValaTypeReference* prerequisite_reference = prerequisite_reference_it->data;
			{
				vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (prerequisite_reference))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
			}
		}
		(__temp33 == NULL ? NULL : (__temp33 = (g_list_free (__temp33), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp35 = NULL;
	ValaSymbol* __temp34 = NULL;
	self->priv->current_symbol = (__temp35 = (__temp34 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp34 == NULL ? NULL : g_object_ref (__temp34))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp35);
}


static void vala_semantic_analyzer_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
	if (!vala_source_file_get_pkg (self->priv->current_source_file)) {
		if (vala_constant_get_initializer (c) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (c), TRUE), vala_code_node_get_error (VALA_CODE_NODE (c)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (c)), "A const field requires a initializer to be provided");
		}
	}
}


static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (f->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
		if (vala_type_reference_get_data_type (vala_field_get_type_reference (f)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		}
	} else {
		if (vala_type_reference_get_data_type (vala_field_get_type_reference (f)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
}


static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp37 = NULL;
	ValaSymbol* __temp36 = NULL;
	self->priv->current_symbol = (__temp37 = (__temp36 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp36 == NULL ? NULL : g_object_ref (__temp36))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp37);
	ValaTypeReference* __temp39 = NULL;
	ValaTypeReference* __temp38 = NULL;
	self->priv->current_return_type = (__temp39 = (__temp38 = vala_method_get_return_type (m), (__temp38 == NULL ? NULL : g_object_ref (__temp38))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp39);
	if (vala_type_reference_get_data_type (vala_method_get_return_type (m)) != NULL) {
		/* is null if it is void or a reference to a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_method_get_return_type (m)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
	}
}


static void vala_semantic_analyzer_find_base_class_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), vala_method_get_name (m));
	if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
		ValaMethod* base_method = VALA_METHOD (vala_symbol_get_node (sym));
		if (vala_method_get_is_abstract (base_method) || vala_method_get_is_virtual (base_method)) {
			if (!vala_method_equals (m, base_method)) {
				(vala_code_node_set_error (VALA_CODE_NODE (m), TRUE), vala_code_node_get_error (VALA_CODE_NODE (m)));
				char* __temp42 = NULL;
				char* __temp41 = NULL;
				char* __temp40 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp42 = g_strdup_printf ("Return type and/or parameters of overriding method `%s' do not match overridden method `%s'.", (__temp40 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp41 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))));
				(__temp42 == NULL ? NULL : (__temp42 = (g_free (__temp42), NULL)));
				(__temp41 == NULL ? NULL : (__temp41 = (g_free (__temp41), NULL)));
				(__temp40 == NULL ? NULL : (__temp40 = (g_free (__temp40), NULL)));
				return;
			}
			(vala_method_set_base_method (m, base_method), vala_method_get_base_method (m));
			return;
		}
	}
	if (vala_class_get_base_class (cl) != NULL) {
		vala_semantic_analyzer_find_base_class_method (self, m, vala_class_get_base_class (cl));
	}
}


static void vala_semantic_analyzer_find_base_interface_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	g_return_if_fail (VALA_IS_CLASS (cl));
	{
		GList* __temp46 = NULL;
		__temp46 = vala_class_get_base_types (cl);
		GList* type_it;
		for (type_it = __temp46; type_it != NULL; type_it = type_it->next) {
			ValaTypeReference* type = type_it->data;
			{
				/* FIXME report error if multiple possible base methods are found*/
				if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (type))) {
					ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (type))), vala_method_get_name (m));
					if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
						ValaMethod* base_method = VALA_METHOD (vala_symbol_get_node (sym));
						if (vala_method_get_is_abstract (base_method)) {
							if (!vala_method_equals (m, base_method)) {
								(vala_code_node_set_error (VALA_CODE_NODE (m), TRUE), vala_code_node_get_error (VALA_CODE_NODE (m)));
								char* __temp45 = NULL;
								char* __temp44 = NULL;
								char* __temp43 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp45 = g_strdup_printf ("Return type and/or parameters of overriding method `%s' do not match overridden method `%s'.", (__temp43 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp44 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))));
								(__temp45 == NULL ? NULL : (__temp45 = (g_free (__temp45), NULL)));
								(__temp44 == NULL ? NULL : (__temp44 = (g_free (__temp44), NULL)));
								(__temp43 == NULL ? NULL : (__temp43 = (g_free (__temp43), NULL)));
								return;
							}
							(vala_method_set_base_interface_method (m, base_method), vala_method_get_base_interface_method (m));
							return;
						}
					}
				}
			}
		}
		(__temp46 == NULL ? NULL : (__temp46 = (g_list_free (__temp46), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp48 = NULL;
	ValaSymbol* __temp47 = NULL;
	self->priv->current_symbol = (__temp48 = (__temp47 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp47 == NULL ? NULL : g_object_ref (__temp47))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp48);
	ValaTypeReference* __temp49 = NULL;
	self->priv->current_return_type = (__temp49 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp49);
	if (vala_symbol_get_parent_symbol (self->priv->current_symbol) != NULL && VALA_IS_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol)))) {
		/* lambda expressions produce nested methods */
		ValaMethod* up_method = VALA_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol)));
		ValaTypeReference* __temp51 = NULL;
		ValaTypeReference* __temp50 = NULL;
		self->priv->current_return_type = (__temp51 = (__temp50 = vala_method_get_return_type (up_method), (__temp50 == NULL ? NULL : g_object_ref (__temp50))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp51);
	}
	if (VALA_IS_CLASS (vala_symbol_get_node (self->priv->current_symbol))) {
		if (!(VALA_IS_CREATION_METHOD (m))) {
			vala_semantic_analyzer_find_base_interface_method (self, m, VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol)));
			if (vala_method_get_is_virtual (m) || vala_method_get_overrides (m)) {
				vala_semantic_analyzer_find_base_class_method (self, m, VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol)));
				if (vala_method_get_base_method (m) == NULL) {
					char* __temp53 = NULL;
					char* __temp52 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp53 = g_strdup_printf ("%s: no suitable method found to override", (__temp52 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
					(__temp53 == NULL ? NULL : (__temp53 = (g_free (__temp53), NULL)));
					(__temp52 == NULL ? NULL : (__temp52 = (g_free (__temp52), NULL)));
				}
			}
		}
	} else {
		if (VALA_IS_STRUCT (vala_symbol_get_node (self->priv->current_symbol))) {
			if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m) || vala_method_get_overrides (m)) {
				char* __temp55 = NULL;
				char* __temp54 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp55 = g_strdup_printf ("A struct member `%s' cannot be marked as override, virtual, or abstract", (__temp54 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
				(__temp55 == NULL ? NULL : (__temp55 = (g_free (__temp55), NULL)));
				(__temp54 == NULL ? NULL : (__temp54 = (g_free (__temp54), NULL)));
				return;
			}
		}
	}
}


static void vala_semantic_analyzer_real_visit_begin_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CREATION_METHOD (m));
	ValaTypeReference* __temp56 = NULL;
	(vala_method_set_return_type (VALA_METHOD (m), (__temp56 = vala_type_reference_new ())), vala_method_get_return_type (VALA_METHOD (m)));
	(__temp56 == NULL ? NULL : (__temp56 = (g_object_unref (__temp56), NULL)));
	(vala_type_reference_set_data_type (vala_method_get_return_type (VALA_METHOD (m)), VALA_DATA_TYPE (vala_symbol_get_node (self->priv->current_symbol))), vala_type_reference_get_data_type (vala_method_get_return_type (VALA_METHOD (m))));
	(vala_type_reference_set_transfers_ownership (vala_method_get_return_type (VALA_METHOD (m)), TRUE), vala_type_reference_get_transfers_ownership (vala_method_get_return_type (VALA_METHOD (m))));
	if (VALA_IS_CLASS (vala_symbol_get_node (self->priv->current_symbol))) {
		/* check for floating reference*/
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol));
		while (cl != NULL) {
			if (VALA_DATA_TYPE (cl) == self->priv->initially_unowned_type) {
				(vala_type_reference_set_floating_reference (vala_method_get_return_type (VALA_METHOD (m)), TRUE), vala_type_reference_get_floating_reference (vala_method_get_return_type (VALA_METHOD (m))));
				break;
			}
			cl = vala_class_get_base_class (cl);
		}
	}
	if (vala_method_get_body (VALA_METHOD (m)) != NULL) {
		(vala_statement_set_construction (VALA_STATEMENT (vala_method_get_body (VALA_METHOD (m))), TRUE), vala_statement_get_construction (VALA_STATEMENT (vala_method_get_body (VALA_METHOD (m)))));
	}
	ValaSymbol* __temp58 = NULL;
	ValaSymbol* __temp57 = NULL;
	self->priv->current_symbol = (__temp58 = (__temp57 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp57 == NULL ? NULL : g_object_ref (__temp57))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp58);
	ValaTypeReference* __temp60 = NULL;
	ValaTypeReference* __temp59 = NULL;
	self->priv->current_return_type = (__temp60 = (__temp59 = vala_method_get_return_type (VALA_METHOD (m)), (__temp59 == NULL ? NULL : g_object_ref (__temp59))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp60);
}


static void vala_semantic_analyzer_real_visit_end_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CREATION_METHOD (m));
	vala_code_visitor_visit_end_method (VALA_CODE_VISITOR (self), VALA_METHOD (m));
	if (vala_method_get_body (VALA_METHOD (m)) != NULL) {
		int n_params = 0;
		{
			GList* __temp61 = NULL;
			__temp61 = vala_block_get_statements (vala_method_get_body (VALA_METHOD (m)));
			GList* stmt_it;
			for (stmt_it = __temp61; stmt_it != NULL; stmt_it = stmt_it->next) {
				ValaStatement* stmt = stmt_it->data;
				{
					int params = vala_statement_get_number_of_set_construction_parameters (stmt);
					if (params == -1) {
						(vala_code_node_set_error (VALA_CODE_NODE (m), TRUE), vala_code_node_get_error (VALA_CODE_NODE (m)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "type creation methods only allow property assignment statements");
						return;
					}
					n_params = n_params + (params);
					(vala_statement_set_construction (stmt, TRUE), vala_statement_get_construction (stmt));
				}
			}
			(__temp61 == NULL ? NULL : (__temp61 = (g_list_free (__temp61), NULL)));
		}
		(vala_creation_method_set_n_construction_params (m, n_params), vala_creation_method_get_n_construction_params (m));
	}
}


static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
	/* special treatment for construct formal parameters used in creation methods */
	if (vala_formal_parameter_get_construct_parameter (p)) {
		if (!(VALA_IS_CREATION_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (p))))))) {
			(vala_code_node_set_error (VALA_CODE_NODE (p), TRUE), vala_code_node_get_error (VALA_CODE_NODE (p)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (p)), "construct parameters are only allowed in type creation methods");
			return;
		}
		ValaBlock* method_body = vala_method_get_body (VALA_METHOD ((VALA_CREATION_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (p))))))));
		ValaMemberAccess* left = vala_member_access_new_simple (vala_formal_parameter_get_name (p), NULL);
		ValaMemberAccess* right = vala_member_access_new_simple (vala_formal_parameter_get_name (p), NULL);
		/* try to lookup the requested property */
		ValaSymbol* prop_sym = vala_semantic_analyzer_symbol_lookup_inherited (self, vala_code_node_get_symbol (VALA_CODE_NODE (self->priv->current_class)), vala_formal_parameter_get_name (p));
		if (!(VALA_IS_PROPERTY (vala_symbol_get_node (prop_sym)))) {
			(vala_code_node_set_error (VALA_CODE_NODE (p), TRUE), vala_code_node_get_error (VALA_CODE_NODE (p)));
			char* __temp63 = NULL;
			char* __temp62 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (p)), (__temp63 = g_strdup_printf ("class `%s' does not contain a property named `%s'", (__temp62 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (self->priv->current_class)))), vala_formal_parameter_get_name (p))));
			(__temp63 == NULL ? NULL : (__temp63 = (g_free (__temp63), NULL)));
			(__temp62 == NULL ? NULL : (__temp62 = (g_free (__temp62), NULL)));
			(left == NULL ? NULL : (left = (g_object_unref (left), NULL)));
			(right == NULL ? NULL : (right = (g_object_unref (right), NULL)));
			return;
		}
		(vala_expression_set_symbol_reference (VALA_EXPRESSION (left), prop_sym), vala_expression_get_symbol_reference (VALA_EXPRESSION (left)));
		(vala_expression_set_symbol_reference (VALA_EXPRESSION (right), vala_code_node_get_symbol (VALA_CODE_NODE (p))), vala_expression_get_symbol_reference (VALA_EXPRESSION (right)));
		ValaExpressionStatement* __temp64 = NULL;
		vala_block_add_statement (method_body, VALA_STATEMENT ((__temp64 = vala_expression_statement_new (VALA_EXPRESSION (vala_assignment_new (VALA_EXPRESSION (left), VALA_EXPRESSION (right), VALA_ASSIGNMENT_OPERATOR_SIMPLE, NULL)), NULL))));
		(__temp64 == NULL ? NULL : (__temp64 = (g_object_unref (__temp64), NULL)));
		(left == NULL ? NULL : (left = (g_object_unref (left), NULL)));
		(right == NULL ? NULL : (right = (g_object_unref (right), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) != NULL) {
		/* is null if it references a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
}


static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc)))));
	if (vala_property_accessor_get_readable (acc)) {
		ValaTypeReference* __temp65 = NULL;
		self->priv->current_return_type = (__temp65 = g_object_ref (vala_property_get_type_reference (prop)), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp65);
	} else {
		/* void*/
		ValaTypeReference* __temp66 = NULL;
		self->priv->current_return_type = (__temp66 = vala_type_reference_new (), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp66);
	}
}


static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaTypeReference* __temp67 = NULL;
	self->priv->current_return_type = (__temp67 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp67);
}


static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp69 = NULL;
	ValaSymbol* __temp68 = NULL;
	self->priv->current_symbol = (__temp69 = (__temp68 = vala_code_node_get_symbol (VALA_CODE_NODE (c)), (__temp68 == NULL ? NULL : g_object_ref (__temp68))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp69);
}


static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp71 = NULL;
	ValaSymbol* __temp70 = NULL;
	self->priv->current_symbol = (__temp71 = (__temp70 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp70 == NULL ? NULL : g_object_ref (__temp70))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp71);
}


static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp73 = NULL;
	ValaSymbol* __temp72 = NULL;
	self->priv->current_symbol = (__temp73 = (__temp72 = vala_code_node_get_symbol (VALA_CODE_NODE (d)), (__temp72 == NULL ? NULL : g_object_ref (__temp72))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp73);
}


static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp75 = NULL;
	ValaSymbol* __temp74 = NULL;
	self->priv->current_symbol = (__temp75 = (__temp74 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp74 == NULL ? NULL : g_object_ref (__temp74))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp75);
}


static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (n));
}


static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp77 = NULL;
	ValaSymbol* __temp76 = NULL;
	self->priv->current_symbol = (__temp77 = (__temp76 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp76 == NULL ? NULL : g_object_ref (__temp76))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp77);
}


static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	{
		GList* __temp78 = NULL;
		__temp78 = vala_block_get_local_variables (b);
		GList* decl_it;
		for (decl_it = __temp78; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				(vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), FALSE), vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))));
			}
		}
		(__temp78 == NULL ? NULL : (__temp78 = (g_list_free (__temp78), NULL)));
	}
	ValaSymbol* __temp80 = NULL;
	ValaSymbol* __temp79 = NULL;
	self->priv->current_symbol = (__temp80 = (__temp79 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp79 == NULL ? NULL : g_object_ref (__temp79))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp80);
}


static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	if (vala_variable_declarator_get_type_reference (decl) == NULL) {
		/* var type */
		if (vala_variable_declarator_get_initializer (decl) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed without initializer");
			return;
		}
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed with non-typed initializer");
			return;
		}
		ValaTypeReference* __temp81 = NULL;
		(vala_variable_declarator_set_type_reference (decl, (__temp81 = vala_type_reference_copy (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))))), vala_variable_declarator_get_type_reference (decl));
		(__temp81 == NULL ? NULL : (__temp81 = (g_object_unref (__temp81), NULL)));
		(vala_type_reference_set_takes_ownership (vala_variable_declarator_get_type_reference (decl), vala_type_reference_get_transfers_ownership (vala_variable_declarator_get_type_reference (decl))), vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl)));
		(vala_type_reference_set_transfers_ownership (vala_variable_declarator_get_type_reference (decl), FALSE), vala_type_reference_get_transfers_ownership (vala_variable_declarator_get_type_reference (decl)));
	}
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			if (!(VALA_IS_MEMBER_ACCESS (vala_variable_declarator_get_initializer (decl)))) {
				(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
			if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
				ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl))));
				ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
				/* check whether method matches callback type */
				if (!vala_callback_matches_method (cb, m)) {
					(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
					char* __temp84 = NULL;
					char* __temp83 = NULL;
					char* __temp82 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), (__temp84 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp82 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp83 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
					(__temp84 == NULL ? NULL : (__temp84 = (g_free (__temp84), NULL)));
					(__temp83 == NULL ? NULL : (__temp83 = (g_free (__temp83), NULL)));
					(__temp82 == NULL ? NULL : (__temp82 = (g_free (__temp82), NULL)));
					return;
				}
				(vala_expression_set_static_type (vala_variable_declarator_get_initializer (decl), vala_variable_declarator_get_type_reference (decl)), vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)));
			} else {
				(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
		}
		if (vala_semantic_analyzer_get_memory_management (self)) {
			if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)))) {
				/* rhs transfers ownership of the expression */
				if (!vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
					/* lhs doesn't own the value
					 * promote lhs type */
					(vala_type_reference_set_takes_ownership (vala_variable_declarator_get_type_reference (decl), TRUE), vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl)));
				}
			}
		}
	}
	if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaSymbol* __temp85 = NULL;
	(vala_code_node_set_symbol (VALA_CODE_NODE (decl), (__temp85 = vala_symbol_new (VALA_CODE_NODE (decl)))), vala_code_node_get_symbol (VALA_CODE_NODE (decl)));
	(__temp85 == NULL ? NULL : (__temp85 = (g_object_unref (__temp85), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_variable_declarator_get_name (decl), vala_code_node_get_symbol (VALA_CODE_NODE (decl)));
	ValaBlock* block = VALA_BLOCK (vala_symbol_get_node (self->priv->current_symbol));
	vala_block_add_local_variable (block, decl);
	(vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), TRUE), vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))));
}


/**
 * Visit operation called for initializer lists
 *
 * @param list an initializer list
 */
static void vala_semantic_analyzer_real_visit_begin_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (list));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (list)) != NULL && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))))) {
		/* initializer is used as array initializer */
		ValaArray* edt = VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))));
		GList* inits = vala_initializer_list_get_initializers (list);
		int rank = vala_array_get_rank ((VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))))));
		ValaTypeReference* child_type = vala_type_reference_copy (vala_expression_get_expected_type (VALA_EXPRESSION (list)));
		if (rank > 1) {
			(vala_type_reference_set_data_type (child_type, VALA_DATA_TYPE (vala_data_type_get_array (vala_array_get_element_type (edt), rank - 1))), vala_type_reference_get_data_type (child_type));
		} else {
			(vala_type_reference_set_data_type (child_type, vala_array_get_element_type (edt)), vala_type_reference_get_data_type (child_type));
		}
		{
			GList* __temp87 = NULL;
			__temp87 = inits;
			GList* e_it;
			for (e_it = __temp87; e_it != NULL; e_it = e_it->next) {
				ValaExpression* e = e_it->data;
				{
					ValaTypeReference* __temp86 = NULL;
					(vala_expression_set_expected_type (e, (__temp86 = vala_type_reference_copy (child_type))), vala_expression_get_expected_type (e));
					(__temp86 == NULL ? NULL : (__temp86 = (g_object_unref (__temp86), NULL)));
				}
			}
		}
		(inits == NULL ? NULL : (inits = (g_list_free (inits), NULL)));
		(child_type == NULL ? NULL : (child_type = (g_object_unref (child_type), NULL)));
	}
}


/**
 * Visit operation called for initializer lists
 *
 * @param list an initializer list
 */
static void vala_semantic_analyzer_real_visit_end_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (list));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (list)) != NULL && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))))) {
		ValaArray* edt = VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))));
		GList* inits = vala_initializer_list_get_initializers (list);
		int rank = vala_array_get_rank (edt);
		ValaTypeReference* child_type = vala_type_reference_copy (vala_expression_get_expected_type (VALA_EXPRESSION (list)));
		gboolean error = FALSE;
		if (rank > 1) {
			(vala_type_reference_set_data_type (child_type, VALA_DATA_TYPE (vala_data_type_get_array (vala_array_get_element_type (edt), rank - 1))), vala_type_reference_get_data_type (child_type));
			{
				GList* __temp89 = NULL;
				__temp89 = inits;
				GList* e_it;
				for (e_it = __temp89; e_it != NULL; e_it = e_it->next) {
					ValaExpression* e = e_it->data;
					{
						if (vala_expression_get_static_type (e) == NULL) {
							error = TRUE;
							continue;
						}
						if (!(VALA_IS_INITIALIZER_LIST (e))) {
							error = TRUE;
							(vala_code_node_set_error (VALA_CODE_NODE (e), TRUE), vala_code_node_get_error (VALA_CODE_NODE (e)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), "Initializer list expected");
							continue;
						}
						if (!vala_type_reference_equals (vala_expression_get_static_type (e), child_type)) {
							error = TRUE;
							(vala_code_node_set_error (VALA_CODE_NODE (e), TRUE), vala_code_node_get_error (VALA_CODE_NODE (e)));
							char* __temp88 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), (__temp88 = g_strdup_printf ("Expected initializer list of type `%s' but got `%s'", vala_data_type_get_name (vala_type_reference_get_data_type (child_type)), vala_data_type_get_name (vala_type_reference_get_data_type (vala_expression_get_static_type (e))))));
							(__temp88 == NULL ? NULL : (__temp88 = (g_free (__temp88), NULL)));
						}
					}
				}
			}
		} else {
			(vala_type_reference_set_data_type (child_type, vala_array_get_element_type (edt)), vala_type_reference_get_data_type (child_type));
			{
				GList* __temp91 = NULL;
				__temp91 = inits;
				GList* e_it;
				for (e_it = __temp91; e_it != NULL; e_it = e_it->next) {
					ValaExpression* e = e_it->data;
					{
						if (vala_expression_get_static_type (e) == NULL) {
							error = TRUE;
							continue;
						}
						if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (e), child_type)) {
							error = TRUE;
							(vala_code_node_set_error (VALA_CODE_NODE (e), TRUE), vala_code_node_get_error (VALA_CODE_NODE (e)));
							char* __temp90 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), (__temp90 = g_strdup_printf ("Expected initializer of type `%s' but got `%s'", vala_data_type_get_name (vala_type_reference_get_data_type (child_type)), vala_data_type_get_name (vala_type_reference_get_data_type (vala_expression_get_static_type (e))))));
							(__temp90 == NULL ? NULL : (__temp90 = (g_free (__temp90), NULL)));
						}
					}
				}
			}
		}
		if (!error) {
			/* everything seems to be correct */
			(vala_expression_set_static_type (VALA_EXPRESSION (list), vala_expression_get_expected_type (VALA_EXPRESSION (list))), vala_expression_get_static_type (VALA_EXPRESSION (list)));
		}
		(inits == NULL ? NULL : (inits = (g_list_free (inits), NULL)));
		(child_type == NULL ? NULL : (child_type = (g_object_unref (child_type), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	if (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)))) {
		vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Short-living reference");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))) {
		/* if there was an error in the condition, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		return;
	}
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_if_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_if_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_while_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_while_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_for_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_for_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_foreach_statement_get_type_reference (stmt)) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_foreach_statement_get_type_reference (stmt)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaVariableDeclarator* __temp92 = NULL;
	(vala_foreach_statement_set_variable_declarator (stmt, (__temp92 = vala_variable_declarator_new (vala_foreach_statement_get_variable_name (stmt), NULL, NULL))), vala_foreach_statement_get_variable_declarator (stmt));
	(__temp92 == NULL ? NULL : (__temp92 = (g_object_unref (__temp92), NULL)));
	(vala_variable_declarator_set_type_reference (vala_foreach_statement_get_variable_declarator (stmt), vala_foreach_statement_get_type_reference (stmt)), vala_variable_declarator_get_type_reference (vala_foreach_statement_get_variable_declarator (stmt)));
	ValaSymbol* __temp93 = NULL;
	(vala_code_node_set_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt)), (__temp93 = vala_symbol_new (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))))), vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))));
	(__temp93 == NULL ? NULL : (__temp93 = (g_object_unref (__temp93), NULL)));
	vala_symbol_add (vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))), vala_foreach_statement_get_variable_name (stmt), vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))));
}


static void vala_semantic_analyzer_real_visit_end_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (self->priv->current_return_type == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return not allowed in this context");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) == NULL && vala_type_reference_get_data_type (self->priv->current_return_type) != NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return without value in non-void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_type_reference_get_data_type (self->priv->current_return_type) == NULL && vala_type_reference_get_type_parameter (self->priv->current_return_type) == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return with value in void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)), self->priv->current_return_type)) {
		char* __temp96 = NULL;
		char* __temp95 = NULL;
		char* __temp94 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), (__temp96 = g_strdup_printf ("Return: Cannot convert from `%s' to `%s'", (__temp94 = vala_type_reference_to_string (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)))), (__temp95 = vala_type_reference_to_string (self->priv->current_return_type)))));
		(__temp96 == NULL ? NULL : (__temp96 = (g_free (__temp96), NULL)));
		(__temp95 == NULL ? NULL : (__temp95 = (g_free (__temp95), NULL)));
		(__temp94 == NULL ? NULL : (__temp94 = (g_free (__temp94), NULL)));
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt))) && !vala_type_reference_get_transfers_ownership (self->priv->current_return_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return value transfers ownership but method return type hasn't been declared to transfer ownership");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)) != NULL && VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)))) && vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt))) && !vala_type_reference_get_transfers_ownership (self->priv->current_return_type)) {
		vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Local variable with strong reference used as return value and method return type hasn't been declared to transfer ownership");
	}
}


/**
 * Visit operation called for lock statements.
 *
 * @param stmt a lock statement
 */
static void vala_semantic_analyzer_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LOCK_STATEMENT (stmt));
	/* resource must be a member access and denote a Lockable */
	if (!(VALA_IS_MEMBER_ACCESS (vala_lock_statement_get_resource (stmt)) && VALA_IS_LOCKABLE (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))))) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		(vala_code_node_set_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt)), TRUE), vala_code_node_get_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))), "Expression is either not a member access or does not denote a lockable member");
		return;
	}
	/* parent symbol must be the current class */
	if (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))) != VALA_CODE_NODE (self->priv->current_class)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		(vala_code_node_set_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt)), TRUE), vala_code_node_get_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))), "Only members of the current class are lockable");
	}
	vala_lockable_set_lock_used ((VALA_LOCKABLE (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt))))), TRUE);
}


static void vala_semantic_analyzer_real_visit_begin_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ARRAY_CREATION_EXPRESSION (expr));
	if (vala_array_creation_expression_get_initializer_list (expr) != NULL) {
		ValaTypeReference* __temp97 = NULL;
		(vala_expression_set_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr)), (__temp97 = vala_type_reference_copy (vala_array_creation_expression_get_element_type (expr)))), vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr))));
		(__temp97 == NULL ? NULL : (__temp97 = (g_object_unref (__temp97), NULL)));
		(vala_type_reference_set_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr))), VALA_DATA_TYPE (vala_data_type_get_array (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr)))), vala_array_creation_expression_get_rank (expr)))), vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr)))));
	}
}


/* FIXME: add element type to type_argument
*
 * Visit operations called for array creation expresions.
 *
 * @param expr an array creation expression
 */
static void vala_semantic_analyzer_real_visit_end_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ARRAY_CREATION_EXPRESSION (expr));
	int i;
	GList* size = vala_array_creation_expression_get_sizes (expr);
	/* check for errors in the size list */
	if (size != NULL) {
		{
			GList* __temp98 = NULL;
			__temp98 = size;
			GList* e_it;
			for (e_it = __temp98; e_it != NULL; e_it = e_it->next) {
				ValaExpression* e = e_it->data;
				{
					if (vala_expression_get_static_type (e) == NULL) {
						(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
						return;
					} else {
						/* return on previous error */
						if (vala_type_reference_get_data_type (vala_expression_get_static_type (e)) != vala_type_reference_get_data_type (self->priv->int_type)) {
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), "Expected expression of type ´int'");
						}
					}
				}
			}
		}
		if (vala_code_node_get_error (VALA_CODE_NODE (expr))) {
			(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
			return;
		}
	}
	/* check for wrong elements inside the initializer */
	if (vala_array_creation_expression_get_initializer_list (expr) != NULL && vala_expression_get_static_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr))) == NULL) {
		(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
		return;
	}
	/* try to construct the type of the array */
	if (vala_array_creation_expression_get_element_type (expr) == NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Cannot determine the element type of the created array");
		(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
		return;
	}
	ValaTypeReference* __temp99 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp99 = vala_type_reference_copy (vala_array_creation_expression_get_element_type (expr)))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp99 == NULL ? NULL : (__temp99 = (g_object_unref (__temp99), NULL)));
	if (vala_type_reference_get_data_type (vala_array_creation_expression_get_element_type (expr)) != NULL) {
		(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), VALA_DATA_TYPE (vala_data_type_get_array (vala_type_reference_get_data_type (vala_array_creation_expression_get_element_type (expr)), vala_array_creation_expression_get_rank (expr)))), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	} else {
		(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), VALA_DATA_TYPE (vala_type_parameter_get_array (vala_type_reference_get_type_parameter (vala_array_creation_expression_get_element_type (expr)), vala_array_creation_expression_get_rank (expr)))), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	}
	(vala_type_reference_set_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	(vala_type_reference_set_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	vala_type_reference_add_type_argument (vala_expression_get_static_type (VALA_EXPRESSION (expr)), vala_array_creation_expression_get_element_type (expr));
	(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
}


static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BOOLEAN_LITERAL (expr));
	(vala_literal_set_static_type (VALA_LITERAL (expr), self->priv->bool_type), vala_literal_get_static_type (VALA_LITERAL (expr)));
}


static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (expr));
	ValaTypeReference* __temp100 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp100 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp100 == NULL ? NULL : (__temp100 = (g_object_unref (__temp100), NULL)));
	(vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "char")))), vala_type_reference_get_data_type (vala_literal_get_static_type (VALA_LITERAL (expr))));
}


static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (expr));
	ValaTypeReference* __temp101 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp101 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp101 == NULL ? NULL : (__temp101 = (g_object_unref (__temp101), NULL)));
	(vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, vala_integer_literal_get_type_name (expr))))), vala_type_reference_get_data_type (vala_literal_get_static_type (VALA_LITERAL (expr))));
}


static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_REAL_LITERAL (expr));
	ValaTypeReference* __temp102 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp102 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp102 == NULL ? NULL : (__temp102 = (g_object_unref (__temp102), NULL)));
	(vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, vala_real_literal_get_type_name (expr))))), vala_type_reference_get_data_type (vala_literal_get_static_type (VALA_LITERAL (expr))));
}


static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRING_LITERAL (expr));
	ValaTypeReference* __temp103 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp103 = vala_type_reference_copy (self->priv->string_type))), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp103 == NULL ? NULL : (__temp103 = (g_object_unref (__temp103), NULL)));
	(vala_type_reference_set_non_null (vala_literal_get_static_type (VALA_LITERAL (expr)), TRUE), vala_type_reference_get_non_null (vala_literal_get_static_type (VALA_LITERAL (expr))));
}


static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NULL_LITERAL (expr));
	/* empty TypeReference represents null */
	ValaTypeReference* __temp104 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp104 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp104 == NULL ? NULL : (__temp104 = (g_object_unref (__temp104), NULL)));
}


static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LITERAL_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_literal_get_static_type (vala_literal_expression_get_literal (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (node), NULL);
	if (VALA_IS_FIELD (node)) {
		ValaField* f = VALA_FIELD (node);
		return g_object_ref (vala_field_get_type_reference (f));
	} else {
		if (VALA_IS_CONSTANT (node)) {
			ValaConstant* c = VALA_CONSTANT (node);
			return g_object_ref (vala_constant_get_type_reference (c));
		} else {
			if (VALA_IS_PROPERTY (node)) {
				ValaProperty* prop = VALA_PROPERTY (node);
				ValaTypeReference* type = vala_type_reference_copy (vala_property_get_type_reference (prop));
				(vala_type_reference_set_takes_ownership (type, FALSE), vala_type_reference_get_takes_ownership (type));
				return type;
				(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			} else {
				if (VALA_IS_FORMAL_PARAMETER (node)) {
					ValaFormalParameter* p = VALA_FORMAL_PARAMETER (node);
					ValaTypeReference* __temp108 = NULL;
					return (__temp108 = vala_formal_parameter_get_type_reference (p), (__temp108 == NULL ? NULL : g_object_ref (__temp108)));
				} else {
					if (VALA_IS_TYPE_REFERENCE (node)) {
						ValaTypeReference* __temp110 = NULL;
						return (__temp110 = VALA_TYPE_REFERENCE (node), (__temp110 == NULL ? NULL : g_object_ref (__temp110)));
					} else {
						if (VALA_IS_VARIABLE_DECLARATOR (node)) {
							ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (node);
							ValaTypeReference* __temp112 = NULL;
							return (__temp112 = vala_variable_declarator_get_type_reference (decl), (__temp112 == NULL ? NULL : g_object_ref (__temp112)));
						} else {
							if (VALA_IS_ENUM_VALUE (node)) {
								ValaTypeReference* type = vala_type_reference_new ();
								(vala_type_reference_set_data_type (type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (node))))), vala_type_reference_get_data_type (type));
								return type;
								(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
							}
						}
					}
				}
			}
		}
	}
	return VALA_TYPE_REFERENCE (NULL);
}


static ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSemanticAnalyzer* self, ValaSymbol* sym, char* name)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_SYMBOL (sym), NULL);
	g_return_val_if_fail (name != NULL, NULL);
	ValaSymbol* result = vala_symbol_lookup (sym, name);
	if (result != NULL) {
		return result;
	}
	if (VALA_IS_CLASS (vala_symbol_get_node (sym))) {
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (sym));
		{
			GList* __temp118 = NULL;
			__temp118 = vala_class_get_base_types (cl);
			GList* base_type_it;
			for (base_type_it = __temp118; base_type_it != NULL; base_type_it = base_type_it->next) {
				ValaTypeReference* base_type = base_type_it->data;
				{
					result = vala_semantic_analyzer_symbol_lookup_inherited (self, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (base_type))), name);
					if (result != NULL) {
						return result;
					}
				}
			}
			(__temp118 == NULL ? NULL : (__temp118 = (g_list_free (__temp118), NULL)));
		}
	} else {
		if (VALA_IS_STRUCT (vala_symbol_get_node (sym))) {
			ValaStruct* st = VALA_STRUCT (vala_symbol_get_node (sym));
			{
				GList* __temp120 = NULL;
				__temp120 = vala_struct_get_base_types (st);
				GList* base_type_it;
				for (base_type_it = __temp120; base_type_it != NULL; base_type_it = base_type_it->next) {
					ValaTypeReference* base_type = base_type_it->data;
					{
						result = vala_semantic_analyzer_symbol_lookup_inherited (self, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (base_type))), name);
						if (result != NULL) {
							return result;
						}
					}
				}
				(__temp120 == NULL ? NULL : (__temp120 = (g_list_free (__temp120), NULL)));
			}
		} else {
			if (VALA_IS_INTERFACE (vala_symbol_get_node (sym))) {
				ValaInterface* iface = VALA_INTERFACE (vala_symbol_get_node (sym));
				{
					GList* __temp122 = NULL;
					__temp122 = vala_interface_get_prerequisites (iface);
					GList* prerequisite_it;
					for (prerequisite_it = __temp122; prerequisite_it != NULL; prerequisite_it = prerequisite_it->next) {
						ValaTypeReference* prerequisite = prerequisite_it->data;
						{
							result = vala_semantic_analyzer_symbol_lookup_inherited (self, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (prerequisite))), name);
							if (result != NULL) {
								return result;
							}
						}
					}
					(__temp122 == NULL ? NULL : (__temp122 = (g_list_free (__temp122), NULL)));
				}
			}
		}
	}
	return VALA_SYMBOL (NULL);
}


static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_parenthesized_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (sym == NULL || VALA_IS_SYMBOL (sym), NULL);
	while (sym != NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
			return VALA_DATA_TYPE (vala_symbol_get_node (sym));
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return VALA_DATA_TYPE (NULL);
}


static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	ValaSymbol* base_symbol = NULL;
	if (vala_member_access_get_inner (expr) == NULL) {
		base_symbol = self->priv->current_symbol;
		ValaSymbol* sym = self->priv->current_symbol;
		while (sym != NULL && vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_semantic_analyzer_symbol_lookup_inherited (self, sym, vala_member_access_get_member_name (expr))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			sym = vala_symbol_get_parent_symbol (sym);
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			{
				GList* __temp129 = NULL;
				__temp129 = self->priv->current_using_directives;
				GList* ns_it;
				for (ns_it = __temp129; ns_it != NULL; ns_it = ns_it->next) {
					ValaNamespaceReference* ns = ns_it->data;
					{
						ValaSymbol* local_sym = vala_symbol_lookup (vala_namespace_reference_get_namespace_symbol (ns), vala_member_access_get_member_name (expr));
						if (local_sym != NULL) {
							if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) != NULL) {
								(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
								char* __temp128 = NULL;
								char* __temp127 = NULL;
								char* __temp126 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp128 = g_strdup_printf ("`%s' is an ambiguous reference between `%s' and `%s'", vala_member_access_get_member_name (expr), (__temp126 = vala_symbol_get_full_name (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)))), (__temp127 = vala_symbol_get_full_name (local_sym)))));
								(__temp128 == NULL ? NULL : (__temp128 = (g_free (__temp128), NULL)));
								(__temp127 == NULL ? NULL : (__temp127 = (g_free (__temp127), NULL)));
								(__temp126 == NULL ? NULL : (__temp126 = (g_free (__temp126), NULL)));
								return;
							}
							(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), local_sym), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
						}
					}
				}
			}
		}
	} else {
		if (vala_code_node_get_error (VALA_CODE_NODE (vala_member_access_get_inner (expr)))) {
			/* if there was an error in the inner expression, skip this check */
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			return;
		}
		if (VALA_IS_MEMBER_ACCESS (vala_member_access_get_inner (expr)) || VALA_IS_BASE_ACCESS (vala_member_access_get_inner (expr))) {
			base_symbol = vala_expression_get_symbol_reference (vala_member_access_get_inner (expr));
			if (VALA_IS_NAMESPACE (vala_symbol_get_node (base_symbol)) || VALA_IS_DATA_TYPE (vala_symbol_get_node (base_symbol))) {
				(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_symbol_lookup (base_symbol, vala_member_access_get_member_name (expr))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			}
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_expression_get_static_type (vala_member_access_get_inner (expr)) != NULL) {
			base_symbol = vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (expr)))));
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_semantic_analyzer_symbol_lookup_inherited (self, base_symbol, vala_member_access_get_member_name (expr))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
		}
	}
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp131 = NULL;
		char* __temp130 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp131 = g_strdup_printf ("The name `%s' does not exist in the context of `%s'", vala_member_access_get_member_name (expr), (__temp130 = vala_symbol_get_full_name (base_symbol)))));
		(__temp131 == NULL ? NULL : (__temp131 = (g_free (__temp131), NULL)));
		(__temp130 == NULL ? NULL : (__temp130 = (g_free (__temp130), NULL)));
		return;
	}
	ValaCodeNode* member = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
	ValaMemberAccessibility access = VALA_MEMBER_ACCESSIBILITY_PUBLIC;
	if (VALA_IS_FIELD (member)) {
		access = (VALA_FIELD (member))->access;
	} else {
		if (VALA_IS_METHOD (member)) {
			access = (VALA_METHOD (member))->access;
		}
	}
	if (access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
		ValaDataType* target_type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (member))));
		ValaDataType* this_type = vala_semantic_analyzer_find_parent_type (self, self->priv->current_symbol);
		if (target_type != this_type) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp133 = NULL;
			char* __temp132 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp133 = g_strdup_printf ("Access to private member `%s' denied", (__temp132 = vala_symbol_get_full_name (vala_code_node_get_symbol (member))))));
			(__temp133 == NULL ? NULL : (__temp133 = (g_free (__temp133), NULL)));
			(__temp132 == NULL ? NULL : (__temp132 = (g_free (__temp132), NULL)));
			return;
		}
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp134 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp134 = vala_semantic_analyzer_get_static_type_for_node (self, vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)))))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp134 == NULL ? NULL : (__temp134 = (g_object_unref (__temp134), NULL)));
}


static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expression_type), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expected_type), FALSE);
	/* only null is compatible to null */
	if (vala_type_reference_get_data_type (expected_type) == NULL && vala_type_reference_get_type_parameter (expected_type) == NULL) {
		return (vala_type_reference_get_data_type (expression_type) == NULL && vala_type_reference_get_type_parameter (expected_type) == NULL);
	}
	if (vala_type_reference_get_data_type (expression_type) == NULL) {
		/* null can be cast to any reference or array type */
		if (vala_type_reference_get_type_parameter (expected_type) != NULL || vala_data_type_is_reference_type (vala_type_reference_get_data_type (expected_type)) || vala_type_reference_get_reference_to_value_type (expected_type) || VALA_IS_ARRAY (vala_type_reference_get_data_type (expected_type)) || VALA_IS_CALLBACK (vala_type_reference_get_data_type (expected_type)) || vala_type_reference_get_data_type (expected_type) == self->priv->pointer_type) {
			return TRUE;
		}
		return FALSE;
	}
	/* null is not compatible with any other type (i.e. value types) 
	 temporarily ignore type parameters */
	if (vala_type_reference_get_type_parameter (expected_type) != NULL) {
		return TRUE;
	}
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (expression_type)) != VALA_IS_ARRAY (vala_type_reference_get_data_type (expected_type))) {
		return FALSE;
	}
	if (VALA_IS_ENUM (vala_type_reference_get_data_type (expression_type)) && vala_type_reference_get_data_type (expected_type) == vala_type_reference_get_data_type (self->priv->int_type)) {
		return TRUE;
	}
	if (vala_type_reference_get_data_type (expression_type) == vala_type_reference_get_data_type (expected_type)) {
		return TRUE;
	}
	if (VALA_IS_STRUCT (vala_type_reference_get_data_type (expression_type)) && VALA_IS_STRUCT (vala_type_reference_get_data_type (expected_type))) {
		ValaStruct* expr_struct = VALA_STRUCT (vala_type_reference_get_data_type (expression_type));
		ValaStruct* expect_struct = VALA_STRUCT (vala_type_reference_get_data_type (expected_type));
		/* integer types may be implicitly cast to floating point types */
		if (vala_struct_is_integer_type (expr_struct) && vala_struct_is_floating_type (expect_struct)) {
			return TRUE;
		}
		if ((vala_struct_is_integer_type (expr_struct) && vala_struct_is_integer_type (expect_struct)) || (vala_struct_is_floating_type (expr_struct) && vala_struct_is_floating_type (expect_struct))) {
			if (vala_struct_get_rank (expr_struct) <= vala_struct_get_rank (expect_struct)) {
				return TRUE;
			}
		}
	}
	return vala_data_type_is_subtype_of (vala_type_reference_get_data_type (expression_type), vala_type_reference_get_data_type (expected_type));
}


static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))) {
		/* if method resolving didn't succeed, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	ValaSymbol* msym = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr));
	if (msym == NULL) {
		/* if no symbol found, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	GList* params = NULL;
	if (VALA_IS_INVOKABLE (vala_symbol_get_node (msym))) {
		ValaInvokable* m = VALA_INVOKABLE (vala_symbol_get_node (msym));
		if (vala_invokable_is_invokable (m)) {
			GList* __temp145 = NULL;
			params = (__temp145 = vala_invokable_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp145);
		} else {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			return;
		}
	} else {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		return;
	}
	GList* args = vala_invocation_expression_get_argument_list (expr);
	GList* arg_it = args;
	{
		GList* __temp146 = NULL;
		__temp146 = params;
		GList* param_it;
		for (param_it = __temp146; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					break;
				}
				if (arg_it != NULL) {
					ValaExpression* arg = VALA_EXPRESSION (arg_it->data);
					/* store expected type for callback parameters */
					(vala_expression_set_expected_type (arg, vala_formal_parameter_get_type_reference (param)), vala_expression_get_expected_type (arg));
					arg_it = arg_it->next;
				}
			}
		}
	}
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
}


static gboolean vala_semantic_analyzer_check_arguments (ValaSemanticAnalyzer* self, ValaExpression* expr, ValaSymbol* msym, GList* params, GList* args)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), FALSE);
	g_return_val_if_fail (VALA_IS_SYMBOL (msym), FALSE);
	GList* prev_arg_it = NULL;
	GList* arg_it = args;
	gboolean diag = (vala_code_node_get_attribute (vala_symbol_get_node (msym), "Diagnostics") != NULL);
	gboolean ellipsis = FALSE;
	int i = 0;
	{
		GList* __temp154 = NULL;
		__temp154 = params;
		GList* param_it;
		for (param_it = __temp154; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					ellipsis = TRUE;
					break;
				}
				/* header file necessary if we need to cast argument */
				if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != NULL) {
					vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
				}
				if (arg_it == NULL) {
					if (vala_formal_parameter_get_default_expression (param) == NULL) {
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						char* __temp148 = NULL;
						char* __temp147 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp148 = g_strdup_printf ("Too few arguments, method `%s' does not take %d arguments", (__temp147 = vala_symbol_get_full_name (msym)), g_list_length (args))));
						(__temp148 == NULL ? NULL : (__temp148 = (g_free (__temp148), NULL)));
						(__temp147 == NULL ? NULL : (__temp147 = (g_free (__temp147), NULL)));
						return FALSE;
					}
				} else {
					ValaExpression* arg = VALA_EXPRESSION (arg_it->data);
					if (vala_expression_get_static_type (arg) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (arg), vala_formal_parameter_get_type_reference (param))) {
						/* if there was an error in the argument,
						 * i.e. arg.static_type == null, skip type check */
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						char* __temp152 = NULL;
						char* __temp151 = NULL;
						char* __temp150 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp152 = g_strdup_printf ("Argument %d: Cannot convert from `%s' to `%s'", i + 1, (__temp150 = vala_type_reference_to_string (vala_expression_get_static_type (arg))), (__temp151 = vala_type_reference_to_string (vala_formal_parameter_get_type_reference (param))))));
						(__temp152 == NULL ? NULL : (__temp152 = (g_free (__temp152), NULL)));
						(__temp151 == NULL ? NULL : (__temp151 = (g_free (__temp151), NULL)));
						(__temp150 == NULL ? NULL : (__temp150 = (g_free (__temp150), NULL)));
						return FALSE;
					}
					prev_arg_it = arg_it;
					arg_it = arg_it->next;
					i++;
				}
			}
		}
	}
	if (!ellipsis && arg_it != NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp156 = NULL;
		char* __temp155 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp156 = g_strdup_printf ("Too many arguments, method `%s' does not take %d arguments", (__temp155 = vala_symbol_get_full_name (msym)), g_list_length (args))));
		(__temp156 == NULL ? NULL : (__temp156 = (g_free (__temp156), NULL)));
		(__temp155 == NULL ? NULL : (__temp155 = (g_free (__temp155), NULL)));
		return FALSE;
	}
	if (diag && prev_arg_it != NULL) {
		ValaExpression* format_arg = VALA_EXPRESSION (prev_arg_it->data);
		if (VALA_IS_LITERAL_EXPRESSION (format_arg)) {
			ValaStringLiteral* format_lit = VALA_STRING_LITERAL (vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (format_arg))));
			char* __temp158 = NULL;
			(vala_string_literal_set_value (format_lit, (__temp158 = g_strdup_printf ("\"%s:%d: %s", vala_source_file_get_filename (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)))), vala_source_reference_get_first_line (vala_code_node_get_source_reference (VALA_CODE_NODE (expr))), g_utf8_offset_to_pointer (vala_string_literal_get_value (format_lit), 1)))), vala_string_literal_get_value (format_lit));
			(__temp158 == NULL ? NULL : (__temp158 = (g_free (__temp158), NULL)));
		}
	}
	return TRUE;
}


static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (expr))) {
		return;
	}
	ValaSymbol* msym = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr));
	ValaTypeReference* ret_type = NULL;
	GList* params = NULL;
	if (VALA_IS_INVOKABLE (vala_symbol_get_node (msym))) {
		ValaInvokable* m = VALA_INVOKABLE (vala_symbol_get_node (msym));
		ret_type = vala_invokable_get_return_type (m);
		GList* __temp160 = NULL;
		params = (__temp160 = vala_invokable_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp160);
	}
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), ret_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	GList* __temp161 = NULL;
	vala_semantic_analyzer_check_arguments (self, VALA_EXPRESSION (expr), msym, params, (__temp161 = vala_invocation_expression_get_argument_list (expr)));
	(__temp161 == NULL ? NULL : (__temp161 = (g_list_free (__temp161), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (expr));
	if (vala_expression_get_static_type (vala_element_access_get_container (expr)) == NULL) {
		/* don't proceed if a child expression failed */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	/* assign a static_type when possible */
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_element_access_get_container (expr))))) {
		GList* args = vala_type_reference_get_type_arguments (vala_expression_get_static_type (vala_element_access_get_container (expr)));
		if (g_list_length (args) != 1) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: array reference without type arguments");
			(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
			return;
		}
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), VALA_TYPE_REFERENCE (args->data)), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
	} else {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp163 = NULL;
		char* __temp162 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp163 = g_strdup_printf ("The expression `%s' does not denote an Array", (__temp162 = vala_type_reference_to_string (vala_expression_get_static_type (vala_element_access_get_container (expr)))))));
		(__temp163 == NULL ? NULL : (__temp163 = (g_free (__temp163), NULL)));
		(__temp162 == NULL ? NULL : (__temp162 = (g_free (__temp162), NULL)));
	}
	{
		GList* __temp164 = NULL;
		__temp164 = vala_element_access_get_indices (expr);
		GList* e_it;
		for (e_it = __temp164; e_it != NULL; e_it = e_it->next) {
			ValaExpression* e = e_it->data;
			{
				/* check if the index is of type integer 
				 don't proceed if a child expression failed */
				if (vala_expression_get_static_type (e) == NULL) {
					return;
				}
				/* check if the index is of type integer */
				if (vala_type_reference_get_data_type (vala_expression_get_static_type (e)) != vala_type_reference_get_data_type (self->priv->int_type)) {
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), "Expression of type `int' expected");
				}
			}
		}
		(__temp164 == NULL ? NULL : (__temp164 = (g_list_free (__temp164), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_base_access (ValaCodeVisitor* base, ValaBaseAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BASE_ACCESS (expr));
	if (self->priv->current_class == NULL) {
		if (self->priv->current_struct == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Base access invalid outside of class and struct");
			return;
		} else {
			GList* __temp165 = NULL;
			gboolean __temp166;
			if ((__temp166 = g_list_length ((__temp165 = vala_struct_get_base_types (self->priv->current_struct))) != 1, (__temp165 == NULL ? NULL : (__temp165 = (g_list_free (__temp165), NULL))), __temp166)) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp168 = NULL;
				GList* __temp167 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp168 = g_strdup_printf ("Base access invalid without base type %d", g_list_length ((__temp167 = vala_struct_get_base_types (self->priv->current_struct))))));
				(__temp168 == NULL ? NULL : (__temp168 = (g_free (__temp168), NULL)));
				(__temp167 == NULL ? NULL : (__temp167 = (g_list_free (__temp167), NULL)));
				return;
			}
		}
		GList* __temp169 = NULL;
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), g_list_first ((__temp169 = vala_struct_get_base_types (self->priv->current_struct)))->data), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(__temp169 == NULL ? NULL : (__temp169 = (g_list_free (__temp169), NULL)));
	} else {
		ValaTypeReference* __temp170 = NULL;
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp170 = vala_type_reference_new ())), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(__temp170 == NULL ? NULL : (__temp170 = (g_object_unref (__temp170), NULL)));
		(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), VALA_DATA_TYPE (vala_class_get_base_class (self->priv->current_class))), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	}
	(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)))))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
}


static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_POSTFIX_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_postfix_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static void vala_semantic_analyzer_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (expr));
	ValaDataType* type = NULL;
	if (vala_object_creation_expression_get_type_reference (expr) == NULL) {
		if (vala_object_creation_expression_get_member_name (expr) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Incomplete object creation expression");
			return;
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr))) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			return;
		}
		ValaCodeNode* constructor_node = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr))));
		ValaCodeNode* type_node = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr))));
		GList* type_args = vala_member_access_get_type_arguments (vala_object_creation_expression_get_member_name (expr));
		if (VALA_IS_METHOD (constructor_node)) {
			type_node = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (constructor_node)));
			ValaMethod* constructor = VALA_METHOD (constructor_node);
			if (!(VALA_IS_CREATION_METHOD (constructor_node))) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp172 = NULL;
				char* __temp171 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp172 = g_strdup_printf ("`%s' is not a construction method", (__temp171 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (constructor)))))));
				(__temp172 == NULL ? NULL : (__temp172 = (g_free (__temp172), NULL)));
				(__temp171 == NULL ? NULL : (__temp171 = (g_free (__temp171), NULL)));
				(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
				return;
			}
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (constructor))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			GList* __temp173 = NULL;
			type_args = (__temp173 = vala_member_access_get_type_arguments ((VALA_MEMBER_ACCESS (vala_member_access_get_inner (vala_object_creation_expression_get_member_name (expr))))), (type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL))), __temp173);
		}
		if (VALA_IS_CLASS (type_node) || VALA_IS_STRUCT (type_node)) {
			type = VALA_DATA_TYPE (type_node);
		} else {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp175 = NULL;
			char* __temp174 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp175 = g_strdup_printf ("`%s' is not a class or struct", (__temp174 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type)))))));
			(__temp175 == NULL ? NULL : (__temp175 = (g_free (__temp175), NULL)));
			(__temp174 == NULL ? NULL : (__temp174 = (g_free (__temp174), NULL)));
			(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
			return;
		}
		ValaTypeReference* __temp176 = NULL;
		(vala_object_creation_expression_set_type_reference (expr, (__temp176 = vala_type_reference_new ())), vala_object_creation_expression_get_type_reference (expr));
		(__temp176 == NULL ? NULL : (__temp176 = (g_object_unref (__temp176), NULL)));
		(vala_type_reference_set_data_type (vala_object_creation_expression_get_type_reference (expr), type), vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)));
		{
			GList* __temp177 = NULL;
			__temp177 = type_args;
			GList* type_arg_it;
			for (type_arg_it = __temp177; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
				ValaTypeReference* type_arg = type_arg_it->data;
				{
					vala_type_reference_add_type_argument (vala_object_creation_expression_get_type_reference (expr), type_arg);
				}
			}
		}
		(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
	} else {
		type = vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr));
	}
	if (!vala_data_type_is_reference_type (type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp179 = NULL;
		char* __temp178 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp179 = g_strdup_printf ("Can't create instance of value type `%s'", (__temp178 = vala_type_reference_to_string (vala_object_creation_expression_get_type_reference (expr))))));
		(__temp179 == NULL ? NULL : (__temp179 = (g_free (__temp179), NULL)));
		(__temp178 == NULL ? NULL : (__temp178 = (g_free (__temp178), NULL)));
		return;
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp180 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp180 = vala_type_reference_copy (vala_object_creation_expression_get_type_reference (expr)))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp180 == NULL ? NULL : (__temp180 = (g_object_unref (__temp180), NULL)));
	(vala_type_reference_set_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	if (VALA_IS_CLASS (type)) {
		ValaClass* cl = VALA_CLASS (type);
		if (vala_class_get_is_abstract (cl)) {
			(vala_expression_set_static_type (VALA_EXPRESSION (expr), NULL), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp182 = NULL;
			char* __temp181 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp182 = g_strdup_printf ("Can't create instance of abstract class `%s'", (__temp181 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cl)))))));
			(__temp182 == NULL ? NULL : (__temp182 = (g_free (__temp182), NULL)));
			(__temp181 == NULL ? NULL : (__temp181 = (g_free (__temp181), NULL)));
			return;
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_class_get_default_construction_method (cl) != NULL) {
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_default_construction_method (cl)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
		}
		while (cl != NULL) {
			if (VALA_DATA_TYPE (cl) == self->priv->initially_unowned_type) {
				(vala_type_reference_set_floating_reference (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_floating_reference (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
				break;
			}
			cl = vala_class_get_base_class (cl);
		}
	} else {
		if (VALA_IS_STRUCT (type)) {
			ValaStruct* st = VALA_STRUCT (type);
			if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_struct_get_default_construction_method (st) != NULL) {
				(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_struct_get_default_construction_method (st)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			}
		}
	}
	GList* __temp183 = NULL;
	gboolean __temp184;
	if ((__temp184 = vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && g_list_length ((__temp183 = vala_object_creation_expression_get_argument_list (expr))) != 0, (__temp183 == NULL ? NULL : (__temp183 = (g_list_free (__temp183), NULL))), __temp184)) {
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), NULL), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp186 = NULL;
		char* __temp185 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp186 = g_strdup_printf ("No arguments allowed when constructing type `%s'", (__temp185 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type)))))));
		(__temp186 == NULL ? NULL : (__temp186 = (g_free (__temp186), NULL)));
		(__temp185 == NULL ? NULL : (__temp185 = (g_free (__temp185), NULL)));
		return;
	}
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) != NULL) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		GList* __temp188 = NULL;
		GList* __temp187 = NULL;
		vala_semantic_analyzer_check_arguments (self, VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp187 = vala_invokable_get_parameters (VALA_INVOKABLE (m))), (__temp188 = vala_object_creation_expression_get_argument_list (expr)));
		(__temp188 == NULL ? NULL : (__temp188 = (g_list_free (__temp188), NULL)));
		(__temp187 == NULL ? NULL : (__temp187 = (g_list_free (__temp187), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_TYPEOF_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->type_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static gboolean vala_semantic_analyzer_is_numeric_type (ValaSemanticAnalyzer* self, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), FALSE);
	if (!(VALA_IS_STRUCT (vala_type_reference_get_data_type (type)))) {
		return FALSE;
	}
	ValaStruct* st = VALA_STRUCT (vala_type_reference_get_data_type (type));
	return vala_struct_is_integer_type (st) || vala_struct_is_floating_type (st);
}


static gboolean vala_semantic_analyzer_is_integer_type (ValaSemanticAnalyzer* self, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), FALSE);
	if (!(VALA_IS_STRUCT (vala_type_reference_get_data_type (type)))) {
		return FALSE;
	}
	ValaStruct* st = VALA_STRUCT (vala_type_reference_get_data_type (type));
	return vala_struct_is_integer_type (st);
}


static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_unary_expression_get_inner (expr)))) {
		/* if there was an error in the inner expression, skip type check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS || vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
		/* integer or floating point type*/
		if (!vala_semantic_analyzer_is_numeric_type (self, vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp194 = NULL;
			char* __temp193 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp194 = g_strdup_printf ("Operator not supported for `%s'", (__temp193 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
			(__temp194 == NULL ? NULL : (__temp194 = (g_free (__temp194), NULL)));
			(__temp193 == NULL ? NULL : (__temp193 = (g_free (__temp193), NULL)));
			return;
		}
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	} else {
		if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
			/* boolean type*/
			if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_unary_expression_get_inner (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp196 = NULL;
				char* __temp195 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp196 = g_strdup_printf ("Operator not supported for `%s'", (__temp195 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
				(__temp196 == NULL ? NULL : (__temp196 = (g_free (__temp196), NULL)));
				(__temp195 == NULL ? NULL : (__temp195 = (g_free (__temp195), NULL)));
				return;
			}
			(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		} else {
			if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
				/* integer type*/
				if (!vala_semantic_analyzer_is_integer_type (self, vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))) {
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					char* __temp198 = NULL;
					char* __temp197 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp198 = g_strdup_printf ("Operator not supported for `%s'", (__temp197 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
					(__temp198 == NULL ? NULL : (__temp198 = (g_free (__temp198), NULL)));
					(__temp197 == NULL ? NULL : (__temp197 = (g_free (__temp197), NULL)));
					return;
				}
				(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
			} else {
				if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT || vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_DECREMENT) {
					/* integer type*/
					if (!vala_semantic_analyzer_is_integer_type (self, vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))) {
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						char* __temp200 = NULL;
						char* __temp199 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp200 = g_strdup_printf ("Operator not supported for `%s'", (__temp199 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
						(__temp200 == NULL ? NULL : (__temp200 = (g_free (__temp200), NULL)));
						(__temp199 == NULL ? NULL : (__temp199 = (g_free (__temp199), NULL)));
						return;
					}
					ValaMemberAccess* ma = vala_semantic_analyzer_find_member_access (self, vala_unary_expression_get_inner (expr));
					if (ma == NULL) {
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Prefix operators not supported for this expression");
						return;
					}
					ValaMemberAccess* old_value = vala_member_access_new (vala_member_access_get_inner (ma), vala_member_access_get_member_name (ma), NULL);
					ValaBinaryExpression* bin = vala_binary_expression_new ((vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT ? VALA_BINARY_OPERATOR_PLUS : VALA_BINARY_OPERATOR_MINUS), VALA_EXPRESSION (old_value), VALA_EXPRESSION (vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("1", NULL)), NULL)), NULL);
					ValaAssignment* assignment = vala_assignment_new (VALA_EXPRESSION (ma), VALA_EXPRESSION (bin), VALA_ASSIGNMENT_OPERATOR_SIMPLE, NULL);
					vala_code_node_replace (vala_code_node_get_parent_node (VALA_CODE_NODE (expr)), VALA_CODE_NODE (expr), VALA_CODE_NODE (assignment));
					vala_code_node_accept (VALA_CODE_NODE (assignment), VALA_CODE_VISITOR (self));
					(old_value == NULL ? NULL : (old_value = (g_object_unref (old_value), NULL)));
					(bin == NULL ? NULL : (bin = (g_object_unref (bin), NULL)));
					(assignment == NULL ? NULL : (assignment = (g_object_unref (assignment), NULL)));
					return;
					(old_value == NULL ? NULL : (old_value = (g_object_unref (old_value), NULL)));
					(bin == NULL ? NULL : (bin = (g_object_unref (bin), NULL)));
					(assignment == NULL ? NULL : (assignment = (g_object_unref (assignment), NULL)));
				} else {
					if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
						/* value type*/
						(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
					} else {
						if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
							/* reference type*/
							(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
						} else {
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: unsupported unary operator");
							return;
						}
					}
				}
			}
		}
	}
}


static ValaMemberAccess* vala_semantic_analyzer_find_member_access (ValaSemanticAnalyzer* self, ValaExpression* expr)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), NULL);
	if (VALA_IS_PARENTHESIZED_EXPRESSION (expr)) {
		ValaParenthesizedExpression* pe = VALA_PARENTHESIZED_EXPRESSION (expr);
		return vala_semantic_analyzer_find_member_access (self, vala_parenthesized_expression_get_inner (pe));
	}
	if (VALA_IS_MEMBER_ACCESS (expr)) {
		return VALA_MEMBER_ACCESS (expr);
	}
	return VALA_MEMBER_ACCESS (NULL);
}


static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (expr));
	if (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)) == NULL) {
		return;
	}
	/* if type resolving didn't succeed, skip this check 
	 FIXME: check whether cast is allowed*/
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_cast_expression_get_type_reference (expr)), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_BINARY_EXPRESSION (expr), FALSE);
	g_return_val_if_fail (operation != NULL, FALSE);
	if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr)))) {
		char* __temp206 = NULL;
		char* __temp205 = NULL;
		char* __temp204 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp206 = g_strdup_printf ("%s: Cannot convert from `%s' to `%s'", operation, (__temp204 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp205 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
		(__temp206 == NULL ? NULL : (__temp206 = (g_free (__temp206), NULL)));
		(__temp205 == NULL ? NULL : (__temp205 = (g_free (__temp205), NULL)));
		(__temp204 == NULL ? NULL : (__temp204 = (g_free (__temp204), NULL)));
		return FALSE;
	}
	return TRUE;
}


static ValaTypeReference* vala_semantic_analyzer_get_arithmetic_result_type (ValaSemanticAnalyzer* self, ValaTypeReference* left_type, ValaTypeReference* right_type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (left_type), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (right_type), NULL);
	if (!(VALA_IS_STRUCT (vala_type_reference_get_data_type (left_type))) || !(VALA_IS_STRUCT (vala_type_reference_get_data_type (right_type)))) {
		return VALA_TYPE_REFERENCE (NULL);
	}
	/* at least one operand not struct*/
	ValaStruct* left = VALA_STRUCT (vala_type_reference_get_data_type (left_type));
	ValaStruct* right = VALA_STRUCT (vala_type_reference_get_data_type (right_type));
	if ((!vala_struct_is_floating_type (left) && !vala_struct_is_integer_type (left)) || (!vala_struct_is_floating_type (right) && !vala_struct_is_integer_type (right))) {
		return VALA_TYPE_REFERENCE (NULL);
	}
	/* at least one operand not numeric*/
	if (vala_struct_is_floating_type (left) == vala_struct_is_floating_type (right)) {
		/* both operands integer or floating type*/
		if (vala_struct_get_rank (left) >= vala_struct_get_rank (right)) {
			return g_object_ref (left_type);
		} else {
			return g_object_ref (right_type);
		}
	} else {
		/* one integer and one floating type operand*/
		if (vala_struct_is_floating_type (left)) {
			return g_object_ref (left_type);
		} else {
			return g_object_ref (right_type);
		}
	}
}


static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_binary_expression_get_left (expr))) || vala_code_node_get_error (VALA_CODE_NODE (vala_binary_expression_get_right (expr)))) {
		/* if there were any errors in inner expressions, skip type check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
		if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) != vala_type_reference_get_data_type (self->priv->string_type)) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be strings");
			return;
		}
		/* string concatenation: convert to a.concat (b) */
		ValaInvocationExpression* concat_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "concat", NULL)), NULL);
		vala_invocation_expression_add_argument (concat_call, vala_binary_expression_get_right (expr));
		vala_code_node_replace (vala_code_node_get_parent_node (VALA_CODE_NODE (expr)), VALA_CODE_NODE (expr), VALA_CODE_NODE (concat_call));
		vala_code_node_accept (VALA_CODE_NODE (concat_call), VALA_CODE_VISITOR (self));
		(concat_call == NULL ? NULL : (concat_call = (g_object_unref (concat_call), NULL)));
	} else {
		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MINUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MUL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_DIV) {
			ValaTypeReference* __temp215 = NULL;
			(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp215 = vala_semantic_analyzer_get_arithmetic_result_type (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr))))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
			(__temp215 == NULL ? NULL : (__temp215 = (g_object_unref (__temp215), NULL)));
			if (vala_expression_get_static_type (VALA_EXPRESSION (expr)) == NULL) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp218 = NULL;
				char* __temp217 = NULL;
				char* __temp216 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp218 = g_strdup_printf ("Arithmetic operation not supported for types `%s' and `%s'", (__temp216 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))), (__temp217 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))))));
				(__temp218 == NULL ? NULL : (__temp218 = (g_free (__temp218), NULL)));
				(__temp217 == NULL ? NULL : (__temp217 = (g_free (__temp217), NULL)));
				(__temp216 == NULL ? NULL : (__temp216 = (g_free (__temp216), NULL)));
				return;
			}
		} else {
			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MOD || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_LEFT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_RIGHT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_XOR) {
				ValaTypeReference* __temp219 = NULL;
				(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp219 = vala_semantic_analyzer_get_arithmetic_result_type (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr))))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
				(__temp219 == NULL ? NULL : (__temp219 = (g_object_unref (__temp219), NULL)));
				if (vala_expression_get_static_type (VALA_EXPRESSION (expr)) == NULL) {
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					char* __temp222 = NULL;
					char* __temp221 = NULL;
					char* __temp220 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp222 = g_strdup_printf ("Arithmetic operation not supported for types `%s' and `%s'", (__temp220 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))), (__temp221 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))))));
					(__temp222 == NULL ? NULL : (__temp222 = (g_free (__temp222), NULL)));
					(__temp221 == NULL ? NULL : (__temp221 = (g_free (__temp221), NULL)));
					(__temp220 == NULL ? NULL : (__temp220 = (g_free (__temp220), NULL)));
					return;
				}
			} else {
				if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL) {
					if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) == vala_type_reference_get_data_type (self->priv->string_type)) {
						/* string comparison: convert to a.collate (b) OP 0 */
						ValaInvocationExpression* cmp_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "collate", NULL)), NULL);
						vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
						(vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call)), vala_binary_expression_get_left (expr));
						ValaLiteralExpression* __temp223 = NULL;
						(vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp223 = vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("0", NULL)), NULL)))), vala_binary_expression_get_right (expr));
						(__temp223 == NULL ? NULL : (__temp223 = (g_object_unref (__temp223), NULL)));
						vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
						(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
					} else {
						/* TODO: check for integer or floating point type in expr.left */
						if (!vala_semantic_analyzer_check_binary_type (self, expr, "Relational operation")) {
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							return;
						}
					}
					(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
				} else {
					if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
						/* relational operation */
						if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr))) && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr)))) {
							char* __temp226 = NULL;
							char* __temp225 = NULL;
							char* __temp224 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp226 = g_strdup_printf ("Equality operation: `%s' and `%s' are incompatible, comparison would always evaluate to false", (__temp224 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp225 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
							(__temp226 == NULL ? NULL : (__temp226 = (g_free (__temp226), NULL)));
							(__temp225 == NULL ? NULL : (__temp225 = (g_free (__temp225), NULL)));
							(__temp224 == NULL ? NULL : (__temp224 = (g_free (__temp224), NULL)));
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							return;
						}
						if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) == vala_type_reference_get_data_type (self->priv->string_type)) {
							/* string comparison: convert to a.collate (b) OP 0 */
							ValaInvocationExpression* cmp_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "collate", NULL)), NULL);
							vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
							(vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call)), vala_binary_expression_get_left (expr));
							ValaLiteralExpression* __temp227 = NULL;
							(vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp227 = vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("0", NULL)), NULL)))), vala_binary_expression_get_right (expr));
							(__temp227 == NULL ? NULL : (__temp227 = (g_object_unref (__temp227), NULL)));
							vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
							(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
						}
						(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
					} else {
						if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_OR) {
							/* integer type or flags type*/
							(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
						} else {
							if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_OR) {
								if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) != vala_type_reference_get_data_type (self->priv->bool_type) || vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
									(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
									vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be boolean");
								}
								(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
							} else {
								g_assert_not_reached ();
							}
						}
					}
				}
			}
		}
	}
}


static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_TYPE_CHECK (expr));
	if (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)) == NULL) {
		/* if type resolving didn't succeed, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONDITIONAL_EXPRESSION (expr));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_conditional_expression_get_condition (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_conditional_expression_get_condition (expr))), "Condition must be boolean");
		return;
	}
	/* FIXME: use greatest lower bound in the type hierarchy 
	 FIXME: support memory management */
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_conditional_expression_get_true_expression (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	char* result = g_strdup_printf ("__lambda%d", self->priv->next_lambda_id);
	self->priv->next_lambda_id++;
	return result;
	(result == NULL ? NULL : (result = (g_free (result), NULL)));
}


static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	ValaSymbol* sym = self->priv->current_symbol;
	while (sym != NULL) {
		if (VALA_IS_METHOD (vala_symbol_get_node (sym))) {
			return VALA_METHOD (vala_symbol_get_node (sym));
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return VALA_METHOD (NULL);
}


static gboolean vala_semantic_analyzer_is_in_constructor (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	ValaSymbol* sym = self->priv->current_symbol;
	while (sym != NULL) {
		if (VALA_IS_CONSTRUCTOR (vala_symbol_get_node (sym))) {
			return TRUE;
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return FALSE;
}


static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (l));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (l)) == NULL || !(VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (l)))))) {
		(vala_code_node_set_error (VALA_CODE_NODE (l), TRUE), vala_code_node_get_error (VALA_CODE_NODE (l)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression not allowed in this context");
		return;
	}
	gboolean in_instance_method = FALSE;
	ValaMethod* current_method = vala_semantic_analyzer_find_current_method (self);
	if (current_method != NULL) {
		in_instance_method = vala_method_get_instance (current_method);
	} else {
		in_instance_method = vala_semantic_analyzer_is_in_constructor (self);
	}
	ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (l))));
	ValaMethod* __temp233 = NULL;
	(vala_lambda_expression_set_method (l, (__temp233 = vala_method_new (vala_semantic_analyzer_get_lambda_name (self), vala_callback_get_return_type (cb), NULL))), vala_lambda_expression_get_method (l));
	(__temp233 == NULL ? NULL : (__temp233 = (g_object_unref (__temp233), NULL)));
	(vala_method_set_instance (vala_lambda_expression_get_method (l), vala_callback_get_instance (cb) && in_instance_method), vala_method_get_instance (vala_lambda_expression_get_method (l)));
	ValaSymbol* __temp234 = NULL;
	(vala_code_node_set_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)), (__temp234 = vala_symbol_new (VALA_CODE_NODE (vala_lambda_expression_get_method (l))))), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))));
	(__temp234 == NULL ? NULL : (__temp234 = (g_object_unref (__temp234), NULL)));
	(vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))), self->priv->current_symbol), vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))));
	GList* lambda_params = vala_lambda_expression_get_parameters (l);
	GList* lambda_param_it = lambda_params;
	{
		GList* __temp236 = NULL;
		__temp236 = vala_callback_get_parameters (cb);
		GList* cb_param_it;
		for (cb_param_it = __temp236; cb_param_it != NULL; cb_param_it = cb_param_it->next) {
			ValaFormalParameter* cb_param = cb_param_it->data;
			{
				if (lambda_param_it == NULL) {
					break;
				}
				/* lambda expressions are allowed to have less parameters */
				char* lambda_param = (char*) lambda_param_it->data;
				ValaFormalParameter* param = vala_formal_parameter_new (lambda_param, vala_formal_parameter_get_type_reference (cb_param), NULL);
				ValaSymbol* __temp235 = NULL;
				(vala_code_node_set_symbol (VALA_CODE_NODE (param), (__temp235 = vala_symbol_new (VALA_CODE_NODE (param)))), vala_code_node_get_symbol (VALA_CODE_NODE (param)));
				(__temp235 == NULL ? NULL : (__temp235 = (g_object_unref (__temp235), NULL)));
				vala_symbol_add (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))), vala_formal_parameter_get_name (param), vala_code_node_get_symbol (VALA_CODE_NODE (param)));
				vala_method_add_parameter (vala_lambda_expression_get_method (l), param);
				lambda_param_it = lambda_param_it->next;
				(param == NULL ? NULL : (param = (g_object_unref (param), NULL)));
			}
		}
		(__temp236 == NULL ? NULL : (__temp236 = (g_list_free (__temp236), NULL)));
	}
	if (lambda_param_it != NULL) {
		/* lambda expressions may not expect more parameters */
		(vala_code_node_set_error (VALA_CODE_NODE (l), TRUE), vala_code_node_get_error (VALA_CODE_NODE (l)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression: too many parameters");
		(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
		return;
	}
	if (vala_lambda_expression_get_expression_body (l) != NULL) {
		ValaBlock* block = vala_block_new (NULL);
		ValaSymbol* __temp237 = NULL;
		(vala_code_node_set_symbol (VALA_CODE_NODE (block), (__temp237 = vala_symbol_new (VALA_CODE_NODE (block)))), vala_code_node_get_symbol (VALA_CODE_NODE (block)));
		(__temp237 == NULL ? NULL : (__temp237 = (g_object_unref (__temp237), NULL)));
		(vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (block)), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))), vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (block))));
		if (vala_type_reference_get_data_type (vala_method_get_return_type (vala_lambda_expression_get_method (l))) != NULL) {
			ValaReturnStatement* __temp238 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp238 = vala_return_statement_new (vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp238 == NULL ? NULL : (__temp238 = (g_object_unref (__temp238), NULL)));
		} else {
			ValaExpressionStatement* __temp239 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp239 = vala_expression_statement_new (vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp239 == NULL ? NULL : (__temp239 = (g_object_unref (__temp239), NULL)));
		}
		(vala_method_set_body (vala_lambda_expression_get_method (l), block), vala_method_get_body (vala_lambda_expression_get_method (l)));
		(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	} else {
		(vala_method_set_body (vala_lambda_expression_get_method (l), vala_lambda_expression_get_statement_body (l)), vala_method_get_body (vala_lambda_expression_get_method (l)));
		(vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_body (vala_lambda_expression_get_method (l)))), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))), vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_body (vala_lambda_expression_get_method (l))))));
	}
	/* lambda expressions should be usable like MemberAccess of a method */
	(vala_expression_set_symbol_reference (VALA_EXPRESSION (l), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (l)));
	(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
}


static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a))) {
		ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
		if (vala_code_node_get_error (VALA_CODE_NODE (ma)) || vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
			return;
		}
		/* if no symbol found, skip this check */
		if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
			ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
			ValaTypeReference* __temp240 = NULL;
			(vala_expression_set_expected_type (vala_assignment_get_right (a), (__temp240 = vala_type_reference_new ())), vala_expression_get_expected_type (vala_assignment_get_right (a)));
			(__temp240 == NULL ? NULL : (__temp240 = (g_object_unref (__temp240), NULL)));
			(vala_type_reference_set_data_type (vala_expression_get_expected_type (vala_assignment_get_right (a)), VALA_DATA_TYPE (vala_signal_get_callback (sig))), vala_type_reference_get_data_type (vala_expression_get_expected_type (vala_assignment_get_right (a))));
		}
	} else {
		if (VALA_IS_ELEMENT_ACCESS (vala_assignment_get_left (a))) {
		} else {
			/* do nothing*/
			(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "unsupported lvalue in assignment");
		}
	}
}


static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (vala_code_node_get_error (VALA_CODE_NODE (a)) || vala_code_node_get_error (VALA_CODE_NODE (vala_assignment_get_left (a))) || vala_code_node_get_error (VALA_CODE_NODE (vala_assignment_get_right (a)))) {
		(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
		return;
	}
	if (vala_assignment_get_operator (a) != VALA_ASSIGNMENT_OPERATOR_SIMPLE && VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a))) {
		/* transform into simple assignment
		 FIXME: only do this if the backend doesn't support
		 the assignment natively*/
		ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
		if (!(VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))))) {
			ValaMemberAccess* old_value = vala_member_access_new (vala_member_access_get_inner (ma), vala_member_access_get_member_name (ma), NULL);
			ValaBinaryExpression* bin = vala_binary_expression_new (VALA_BINARY_OPERATOR_PLUS, VALA_EXPRESSION (old_value), VALA_EXPRESSION (vala_parenthesized_expression_new (vala_assignment_get_right (a), vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))))), NULL);
			if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
				(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_BITWISE_OR), vala_binary_expression_get_operator (bin));
			} else {
				if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
					(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_BITWISE_AND), vala_binary_expression_get_operator (bin));
				} else {
					if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
						(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_BITWISE_XOR), vala_binary_expression_get_operator (bin));
					} else {
						if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_ADD) {
							(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_PLUS), vala_binary_expression_get_operator (bin));
						} else {
							if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SUB) {
								(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_MINUS), vala_binary_expression_get_operator (bin));
							} else {
								if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_MUL) {
									(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_MUL), vala_binary_expression_get_operator (bin));
								} else {
									if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_DIV) {
										(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_DIV), vala_binary_expression_get_operator (bin));
									} else {
										if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
											(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_MOD), vala_binary_expression_get_operator (bin));
										} else {
											if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
												(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_SHIFT_LEFT), vala_binary_expression_get_operator (bin));
											} else {
												if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
													(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_SHIFT_RIGHT), vala_binary_expression_get_operator (bin));
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			(vala_assignment_set_right (a, VALA_EXPRESSION (bin)), vala_assignment_get_right (a));
			vala_code_node_accept (VALA_CODE_NODE (vala_assignment_get_right (a)), VALA_CODE_VISITOR (self));
			(vala_assignment_set_operator (a, VALA_ASSIGNMENT_OPERATOR_SIMPLE), vala_assignment_get_operator (a));
			(old_value == NULL ? NULL : (old_value = (g_object_unref (old_value), NULL)));
			(bin == NULL ? NULL : (bin = (g_object_unref (bin), NULL)));
		}
	}
	if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a))) {
		ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
		if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
			ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
			if (vala_expression_get_symbol_reference (vala_assignment_get_right (a)) == NULL) {
				(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "unsupported expression for signal handler");
				return;
			}
			ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_right (a))));
			if (vala_method_get_instance (m) && m->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
				/* TODO: generate wrapper function */
				(vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE), vala_code_node_get_error (VALA_CODE_NODE (ma)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "public instance methods not yet supported as signal handlers");
				return;
			}
			if (vala_method_get_instance (m)) {
				/* instance signal handlers must have the self
				 * parameter at the end
				 * do not use G_CONNECT_SWAPPED as this would
				 * rearrange the parameters for instance
				 * methods and non-instance methods
				 */
				(vala_method_set_instance_last (m, TRUE), vala_method_get_instance_last (m));
			}
		} else {
			if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
				ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
				if (vala_property_get_set_accessor (prop) == NULL) {
					(vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE), vala_code_node_get_error (VALA_CODE_NODE (ma)));
					char* __temp242 = NULL;
					char* __temp241 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (ma)), (__temp242 = g_strdup_printf ("Property `%s' is read-only", (__temp241 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))));
					(__temp242 == NULL ? NULL : (__temp242 = (g_free (__temp242), NULL)));
					(__temp241 == NULL ? NULL : (__temp241 = (g_free (__temp241), NULL)));
					return;
				}
			} else {
				if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
					ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
					ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
					if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
						ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma))));
						ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
						/* check whether method matches callback type */
						if (!vala_callback_matches_method (cb, m)) {
							(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
							char* __temp245 = NULL;
							char* __temp244 = NULL;
							char* __temp243 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp245 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp243 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp244 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
							(__temp245 == NULL ? NULL : (__temp245 = (g_free (__temp245), NULL)));
							(__temp244 == NULL ? NULL : (__temp244 = (g_free (__temp244), NULL)));
							(__temp243 == NULL ? NULL : (__temp243 = (g_free (__temp243), NULL)));
							return;
						}
						(vala_expression_set_static_type (vala_assignment_get_right (a), vala_variable_declarator_get_type_reference (decl)), vala_expression_get_static_type (vala_assignment_get_right (a)));
					} else {
						(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
						return;
					}
				} else {
					if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
						ValaField* f = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
						ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
						if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))) {
							ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma))));
							ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)));
							/* check whether method matches callback type */
							if (!vala_callback_matches_method (cb, m)) {
								(vala_code_node_set_error (VALA_CODE_NODE (f), TRUE), vala_code_node_get_error (VALA_CODE_NODE (f)));
								char* __temp248 = NULL;
								char* __temp247 = NULL;
								char* __temp246 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp248 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp246 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp247 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
								(__temp248 == NULL ? NULL : (__temp248 = (g_free (__temp248), NULL)));
								(__temp247 == NULL ? NULL : (__temp247 = (g_free (__temp247), NULL)));
								(__temp246 == NULL ? NULL : (__temp246 = (g_free (__temp246), NULL)));
								return;
							}
							(vala_expression_set_static_type (vala_assignment_get_right (a), vala_field_get_type_reference (f)), vala_expression_get_static_type (vala_assignment_get_right (a)));
						} else {
							(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
							return;
						}
					} else {
						if (vala_expression_get_static_type (vala_assignment_get_left (a)) != NULL && vala_expression_get_static_type (vala_assignment_get_right (a)) != NULL) {
							if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_assignment_get_right (a)), vala_expression_get_static_type (vala_assignment_get_left (a)))) {
								/* if there was an error on either side,
								 * i.e. a.{left|right}.static_type == null, skip type check */
								(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
								char* __temp251 = NULL;
								char* __temp250 = NULL;
								char* __temp249 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp251 = g_strdup_printf ("Assignment: Cannot convert from `%s' to `%s'", (__temp249 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_right (a)))), (__temp250 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_left (a)))))));
								(__temp251 == NULL ? NULL : (__temp251 = (g_free (__temp251), NULL)));
								(__temp250 == NULL ? NULL : (__temp250 = (g_free (__temp250), NULL)));
								(__temp249 == NULL ? NULL : (__temp249 = (g_free (__temp249), NULL)));
								return;
							}
							if (vala_semantic_analyzer_get_memory_management (self)) {
								if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
									/* rhs transfers ownership of the expression */
									if (!vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
										/* lhs doesn't own the value
										 * promote lhs type if it is a local variable
										 * error if it's not a local variable */
										if (!(VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))))) {
											vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Invalid assignment from owned expression to unowned variable");
										}
										(vala_type_reference_set_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)), TRUE), vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a))));
									}
								} else {
									if (vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
									}
								}
							}
						}
					}
				}
			}
		}
	} else {
		/* lhs wants to own the value
		 * rhs doesn't transfer the ownership
		 * code generator needs to add reference
		 * increment calls */
		if (VALA_IS_ELEMENT_ACCESS (vala_assignment_get_left (a))) {
			ValaElementAccess* ea = VALA_ELEMENT_ACCESS (vala_assignment_get_left (a));
			if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_assignment_get_right (a)), vala_expression_get_static_type (vala_assignment_get_left (a)))) {
				/* if there was an error on either side,
				 * i.e. a.{left|right}.static_type == null, skip type check */
				(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
				char* __temp254 = NULL;
				char* __temp253 = NULL;
				char* __temp252 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp254 = g_strdup_printf ("Assignment: Cannot convert from `%s' to `%s'", (__temp252 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_right (a)))), (__temp253 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_left (a)))))));
				(__temp254 == NULL ? NULL : (__temp254 = (g_free (__temp254), NULL)));
				(__temp253 == NULL ? NULL : (__temp253 = (g_free (__temp253), NULL)));
				(__temp252 == NULL ? NULL : (__temp252 = (g_free (__temp252), NULL)));
				return;
			}
			if (vala_semantic_analyzer_get_memory_management (self)) {
				if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
					/* rhs transfers ownership of the expression */
					GList* args = vala_type_reference_get_type_arguments (vala_expression_get_static_type (vala_element_access_get_container (ea)));
					if (g_list_length (args) != 1) {
						(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (ea)), "internal error: array reference without type arguments");
						(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
						return;
					}
					ValaTypeReference* element_type = VALA_TYPE_REFERENCE (args->data);
					if (!vala_type_reference_get_takes_ownership (element_type)) {
						/* lhs doesn't own the value
						 * promote lhs type if it is a local variable
						 * error if it's not a local variable */
						if (!(VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_element_access_get_container (ea)))))) {
							(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Invalid assignment from owned expression to unowned variable");
							(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
							return;
						}
						(vala_type_reference_set_takes_ownership (element_type, TRUE), vala_type_reference_get_takes_ownership (element_type));
					}
					(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
				} else {
					if (vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
					}
				}
			}
		} else {
			return;
		}
	}
	/* lhs wants to own the value
	 * rhs doesn't transfer the ownership
	 * code generator needs to add reference
	 * increment calls */
	(vala_expression_set_static_type (VALA_EXPRESSION (a), vala_expression_get_static_type (vala_assignment_get_left (a))), vala_expression_get_static_type (VALA_EXPRESSION (a)));
}


gboolean vala_semantic_analyzer_get_memory_management (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	return self->priv->_memory_management;
}


void vala_semantic_analyzer_set_memory_management (ValaSemanticAnalyzer* self, gboolean value)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	self->priv->_memory_management = value;
}


static void vala_semantic_analyzer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		g_value_set_boolean (value, vala_semantic_analyzer_get_memory_management (self));
		break;
	}
}


static void vala_semantic_analyzer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		vala_semantic_analyzer_set_memory_management (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_semantic_analyzer_class_init (ValaSemanticAnalyzerClass * klass)
{
	vala_semantic_analyzer_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaSemanticAnalyzerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_semantic_analyzer_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_semantic_analyzer_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_semantic_analyzer_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_semantic_analyzer_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_semantic_analyzer_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_semantic_analyzer_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_semantic_analyzer_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_semantic_analyzer_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_semantic_analyzer_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_semantic_analyzer_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_semantic_analyzer_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_semantic_analyzer_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_semantic_analyzer_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_semantic_analyzer_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_semantic_analyzer_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_semantic_analyzer_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_semantic_analyzer_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_creation_method = vala_semantic_analyzer_real_visit_begin_creation_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_creation_method = vala_semantic_analyzer_real_visit_end_creation_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_semantic_analyzer_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_semantic_analyzer_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property_accessor = vala_semantic_analyzer_real_visit_begin_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_semantic_analyzer_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_constructor = vala_semantic_analyzer_real_visit_begin_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_semantic_analyzer_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_destructor = vala_semantic_analyzer_real_visit_begin_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_destructor = vala_semantic_analyzer_real_visit_end_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_named_argument = vala_semantic_analyzer_real_visit_named_argument;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_semantic_analyzer_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_semantic_analyzer_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_semantic_analyzer_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_initializer_list = vala_semantic_analyzer_real_visit_begin_initializer_list;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_initializer_list = vala_semantic_analyzer_real_visit_end_initializer_list;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_semantic_analyzer_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_semantic_analyzer_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_semantic_analyzer_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_semantic_analyzer_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_foreach_statement = vala_semantic_analyzer_real_visit_begin_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_return_statement = vala_semantic_analyzer_real_visit_end_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_lock_statement = vala_semantic_analyzer_real_visit_lock_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_array_creation_expression = vala_semantic_analyzer_real_visit_begin_array_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_array_creation_expression = vala_semantic_analyzer_real_visit_end_array_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_boolean_literal = vala_semantic_analyzer_real_visit_boolean_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_character_literal = vala_semantic_analyzer_real_visit_character_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_integer_literal = vala_semantic_analyzer_real_visit_integer_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_real_literal = vala_semantic_analyzer_real_visit_real_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_string_literal = vala_semantic_analyzer_real_visit_string_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_null_literal = vala_semantic_analyzer_real_visit_null_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_literal_expression = vala_semantic_analyzer_real_visit_literal_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_parenthesized_expression = vala_semantic_analyzer_real_visit_parenthesized_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_semantic_analyzer_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_invocation_expression = vala_semantic_analyzer_real_visit_begin_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_semantic_analyzer_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_element_access = vala_semantic_analyzer_real_visit_element_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_base_access = vala_semantic_analyzer_real_visit_base_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_postfix_expression = vala_semantic_analyzer_real_visit_postfix_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_object_creation_expression = vala_semantic_analyzer_real_visit_end_object_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_typeof_expression = vala_semantic_analyzer_real_visit_typeof_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_unary_expression = vala_semantic_analyzer_real_visit_unary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_cast_expression = vala_semantic_analyzer_real_visit_cast_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_semantic_analyzer_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_check = vala_semantic_analyzer_real_visit_type_check;
	VALA_CODE_VISITOR_CLASS (klass)->visit_conditional_expression = vala_semantic_analyzer_real_visit_conditional_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_lambda_expression = vala_semantic_analyzer_real_visit_begin_lambda_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_assignment = vala_semantic_analyzer_real_visit_begin_assignment;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_semantic_analyzer_real_visit_end_assignment;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT, g_param_spec_boolean ("memory-management", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_semantic_analyzer_init (ValaSemanticAnalyzer * self)
{
	self->priv = VALA_SEMANTIC_ANALYZER_GET_PRIVATE (self);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_dispose (GObject * obj)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (obj);
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL)));
	(self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL)));
	(self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL)));
	(self->priv->current_struct == NULL ? NULL : (self->priv->current_struct = (g_object_unref (self->priv->current_struct), NULL)));
	(self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL)));
	(self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL)));
	(self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL)));
	(self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL)));
	(self->priv->type_type == NULL ? NULL : (self->priv->type_type = (g_object_unref (self->priv->type_type), NULL)));
	(self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL)));
	(self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL)));
	ValaSemanticAnalyzerClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SEMANTIC_ANALYZER_CLASS (g_type_class_peek (VALA_TYPE_SEMANTIC_ANALYZER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_semantic_analyzer_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSemanticAnalyzerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_semantic_analyzer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSemanticAnalyzer), 0, (GInstanceInitFunc) vala_semantic_analyzer_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaSemanticAnalyzer", &g_define_type_info, 0);
	}
	return g_define_type_id;
}




